/****************************************************************************
*
* File              : SimpleKeyboard.c
* Compiler          : WinAVR 4.11
* Revision          : $Revision: 0.10 $
* Date              : $Date: Tuesday, April 30, 2013 $
* Updated by        : $Author: Frank Brennecke $
*
* Support mail      : frank@brennecle.org
*
* Supported devices : ATmega88
*
* AppNote           : ?
* 
* Description       : Simple Keyboard 
*                     
*
*
****************************************************************************/


/****************************************************************************
  Libraries
****************************************************************************/

#include "SimpleKeyboard.h"

/****************************************************************************
  Global variable
****************************************************************************/

unsigned char LastRegisterValue = 0;

/****************************************************************************
  Interrupts
****************************************************************************/

ISR(PCINT0_vect)  // Ausgelöst durch einen Tastendruck
{
    unsigned char RegisterValue;
// Prüfen, welches Register welchen Status hat
	Keys = 0;
    RegisterValue = KEYBOARD_INPUT & (BUTTON_OK | BUTTON_LEFT | BUTTON_RIGHT);
	if (RegisterValue != LastRegisterValue)  // Knopfauswahl hat sich geändert
	{
		LastRegisterValue = RegisterValue;
		if ((RegisterValue & BUTTON_OK) == 0)
			Keys = KEY_OK;
		else if ((RegisterValue & BUTTON_LEFT) == 0)
			Keys = KEY_LEFT;
		else if ((RegisterValue & BUTTON_RIGHT) == 0)
			Keys = KEY_RIGHT;
		else
			Keys = 0;
	}		
}

/****************************************************************************
 Function declarations
 ****************************************************************************/
void Keyboard_Init(void)
{
	DDRB = 0x00;		// Port B: Input bei B0, B1, B2 durch 3 Tasten OK, Links und Rechts  
	PORTB = 0x07;		// Pull-up-Widerstände aktivieren 
	Keys = 0;
    // Pin Change
    // Set Pin Change Interrupt Source
    // SetBit(PIN_CHANGE_MASK_0_REGISTER, PIN_CHANGE_ENABLE_MASK_0);
    // Enable Pin Change Interrupt
    // SetBit(PIN_CHANGE_INTERRUPT_CONTROL_REGISTER, PIN_CHANGE_INTERRUPT_ENABLE_0);
}


char GetKeys(void)
{
	unsigned char c, value;
	value = PINB;
	if (value & 0x01)
	{
		if (value & 0x02)
		{
			if (value & 0x04)
			{
				c = 0;
				Keys = 0;
			}	
			else
				c = KEY_RIGHT;
		}		
		else
			c = KEY_LEFT;
	}
	else		
		c = KEY_OK;
	if (c==Keys)	
		return 0;
	else
	{
		Keys = c;
		return Keys;
	}
}
