/****************************************************************************
*
*
* File              : SimpleKeyboard.h
* Compiler          : WinAVR 4.11
* Revision          : $Revision: 0.99 $
* Date              : $Date: Tuesday, April 30, 2013 $
* Updated by        : $Author: Frank Brennecke $
*
* Support mail      : frank@brennecke.org
*
* Supported devices : ATmega88
*
* AppNote           : ?
* 
* Description       : Three-Button Simple Keyboard Interface (Up Down Enter) and Test Mode
*                     
*
*
****************************************************************************/

#ifndef SIMPLEKEYBOARD_H
#define SIMPLEKEYBOARD_H
#endif

/****************************************************************************
  Libraries
****************************************************************************/

#include <avr/io.h> 
#include <avr/interrupt.h> 
#include "usart.h"

/****************************************************************************
  Function definitions
****************************************************************************/


void Keyboard_Init(void);

/****************************************************************************
  Global variable
****************************************************************************/


unsigned char Keys=0;



/****************************************************************************
  Keyboard
****************************************************************************/

// Register Definitions

// I/O
#define KEYBOARD_INPUT                                PINB
#define KEYBOARD_OUTPUT                               PORTB
#define KEYBOARD_DDR                                  DDRB
// Pin Change
#define PIN_CHANGE_INTERRUPT_CONTROL_REGISTER         PCICR
#define PIN_CHANGE_INTERRUPT_FLAG_REGISTER            PCIFR
#define PIN_CHANGE_MASK_0_REGISTER                    PCMSK0

// Bit Definitions
#define BUTTON_LEFT                                   PB1
#define BUTTON_OK	                                  PB0
#define BUTTON_RIGHT                                  PB2

// Key Definition
#define KEY_LEFT                                      1
#define KEY_OK		                                  2
#define KEY_RIGHT                                     3

// Pin Change
#define PIN_CHANGE_INTERRUPT_ENABLE_0 	              PCIE0
#define PIN_CHANGE_INTERRUPT_FLAG_0                   PCIF0
#define PIN_CHANGE_ENABLE_MASK_0                      PCINT0 | PCINT1 | PCINT2 // B0 bis B2


/****************************************************************************
  General Device dependent defines
****************************************************************************/

// Interrupt Vector Definitions
#define PIN_CHANGE_VECTOR_0                                 PCINT0_vect


/****************************************************************************
 Function prototypes
 ****************************************************************************/
void Keyboard_Init(void);

