/****************************************************************************
 *
 *
 * File              : calculator.c
 * Compiler          : WinAVR 4.11
 * Revision          : $Revision: 0.99 $
 * Date              : $Date: Wednesday, May 25, 2013 $
 * Updated by        : $Author: Frank Brennecke $
 *
 * 
 * Supported devices : ATMega644
 * Description       : Simple Calculator for Microcontroller Lab
 *
 ****************************************************************************/
 
#include <math.h> 
#include "mclib.c"


#define Key_1	18
#define Key_2	19
#define Key_3	20
#define Key_4	10
#define Key_5	11
#define Key_6	12
#define Key_7	2
#define Key_8	3
#define Key_9	4
#define Key_0	26
#define Key_KOMMA	28
#define Key_CHS		27

#define Key_DIV		6
#define Key_MULT	14
#define Key_PLUS	30
#define Key_MINUS	22

#define Key_MRECALL	24
#define Key_MMINUS	16
#define Key_MPLUS	8
#define Key_MCLEAR	0
#define Key_CLEAR	7
#define Key_EQUATE	31

#define Key_NULL		255
#define Key_OPERAND1	254
#define Key_OPERAND2	253



#define		CALC_BUFFER_SIZE 8
#define CALC_PLUS 		1
#define CALC_MINUS 		2
#define CALC_DIV		3
#define CALC_MULT		4


#define OPERATOR_PLUS	43
#define OPERATOR_MINUS	45
#define OPERATOR_MULT	42
#define OPERATOR_DIV	47
#define OPERATOR_NULL	32

#define OPERAND1		0
#define OPERAND2		1



void FormatOperand(double Value, char* Buffer)
{
	char NachKommaStellen=0, VorKommaStellen=0;
	double tempValue, Operand;
	
	Operand = Value;
	if (Operand != 0)
		VorKommaStellen = log10(abs(Operand)) + 1;
	else
		VorKommaStellen = 1;
				
	Operand = modf(Operand, &tempValue);
	if (Operand < 0)
		Operand = Operand * -1;
			
	while (Operand > 0.1)
	{
		NachKommaStellen++;
		Operand = Operand * 10;
		Operand= modf(Operand, &tempValue);
		if (NachKommaStellen >= CALC_BUFFER_SIZE-VorKommaStellen)
			break;
	}
	dtostrf(Value, VorKommaStellen+NachKommaStellen+1, NachKommaStellen, Buffer);
}




void Taschenrechner(void)
{
	unsigned char c, selected=MAXBUTTONS, comma=0, current_operand=OPERAND1, key_value, Operator=OPERATOR_NULL, NextOperator=OPERATOR_NULL;
	unsigned char CurrentChar=0;
	double dOperand1, dOperand2, dMemory=0;
	char Operand1_Buffer[CALC_BUFFER_SIZE+2];
	char Operand2_Buffer[CALC_BUFFER_SIZE+2];
// Diese Routine benutzt den Keyboard-Modus ohne Tastenbezeichnungen, um eigene Tasten zu definieren
// Die Tasten werden mittels der Terminal-Zeichenfunktion beschriftet
	Send_Command(MODE_QUIT);  // Aktuellen Modus verlassen
	Send_Command(MODE_KEYBOARD_DRAW);   // Freies Zeichnen  
	Send_Command(KEYBOARD_SWITCHTERMINAL);
	Send_Command(TERMINAL_BKCOLOR);
	Send_Value(WindowBkColor);	// Hintergrundfarbe fr Textausgabe
	
	// Zahlen
	Send_Keyboard_Button_Text("1", Key_1, 0);
	Send_Keyboard_Button_Text("2", Key_2, 0);
	Send_Keyboard_Button_Text("3", Key_3, 0);
	Send_Keyboard_Button_Text("4", Key_4, 0);
	Send_Keyboard_Button_Text("5", Key_5, 0);
	Send_Keyboard_Button_Text("6", Key_6, 0);
	Send_Keyboard_Button_Text("7", Key_7, 0);
	Send_Keyboard_Button_Text("8", Key_8, 0);
	Send_Keyboard_Button_Text("9", Key_9, 0);
	Send_Keyboard_Button_Text("0", Key_0, 0);
	Send_Keyboard_Button_Text(",", Key_KOMMA, 0);
	
	// Rechenfunktionen
	Send_Keyboard_Button_Text("/", Key_DIV, 0);
	Send_Keyboard_Button_Text("*", Key_MULT, 0);
	Send_Keyboard_Button_Text("-", Key_MINUS, 0);
	Send_Keyboard_Button_Text("+", Key_PLUS, 0);
	
	// Speicher
	Send_Keyboard_Button_Text("MC", Key_MCLEAR, 0);
	Send_Keyboard_Button_Text("M+", Key_MPLUS, 0);
	Send_Keyboard_Button_Text("M-", Key_MMINUS, 0);
	Send_Keyboard_Button_Text("MR", Key_MRECALL, 0);
	
	// Sonstiges
	Send_Keyboard_Button_Text("+-", Key_CHS, 0);
	Send_Keyboard_Button_Text("=", Key_EQUATE, 0);
	Send_Keyboard_Button_Text("C", Key_CLEAR, 0);
	
	Send_Command(MODE_END_INTERMEDIATE);
	
	Operand1_Buffer[0] = 0;
	Operand2_Buffer[0] = 0;
	
	for (;;)
	{
		c = Wait_For_Input;	
		
// erst die Touch-Werte auswerten und ggf. den Wert fr c ndern
		selected = Check_Keyboard_Keys(c, selected, OWNERDRAW_KEYBOARD);
		
		if ((c >= SENDKEY_TOUCH_END+44) && (c <= SENDKEY_TOUCH_END + 47))
			break;
			
		
		else if (c== SENDKEY_OK)
			c = selected + SENDKEY_TOUCH_END + 8;
		if ((c >= SENDKEY_TOUCH_END) && (c <= SENDKEY_TOUCH_END + 43))
		{
			c = c - SENDKEY_TOUCH_END - 8;	// Button-Nummer ermitteln
			
			key_value = Key_NULL;
			
			if (c == Key_1)
				key_value = 1;
			else if (c == Key_2)	
				key_value = 2;
			else if (c == Key_3)	
				key_value = 3;
			else if (c == Key_4)	
				key_value = 4;
			else if (c == Key_5)	
				key_value = 5;	
			else if (c == Key_6)	
				key_value = 6;
			else if (c == Key_7)	
				key_value = 7;
			else if (c == Key_8)	
				key_value = 8;
			else if (c == Key_9)	
				key_value = 9;
			else if (c == Key_0)	
				key_value = 0;
			else if (c == Key_CLEAR)
			{
				Operand1_Buffer[0] = 0;
				Operand2_Buffer[0] = 0;
				current_operand=OPERAND1;
				Operator=OPERATOR_NULL;
				NextOperator=OPERATOR_NULL;
				comma = 0;
				dMemory = 0;
				WriteKeyboardCaption(Operand1_Buffer, Operator, Operand2_Buffer);
			}
			else if (c == Key_EQUATE)
			{
				if (current_operand==OPERAND2)
				{
					key_value = Key_EQUATE;
					NextOperator = OPERATOR_NULL;
				}	
			}
			else if (c == Key_PLUS)
			{
				if (current_operand==OPERAND2)
				{
					key_value = Key_EQUATE;
					NextOperator = OPERATOR_PLUS;
				}
				else
				{	
					Operator = OPERATOR_PLUS;
					key_value = Key_OPERAND2;
				}
			}	
			else if (c == Key_MINUS)
			{
				if (current_operand==OPERAND2)
				{
					key_value = Key_EQUATE;
					NextOperator = OPERATOR_MINUS;
				}
				else
				{	
					Operator = OPERATOR_MINUS;
					key_value = Key_OPERAND2;
				}
			}	
			else if (c == Key_MULT)
			{
				if (current_operand==OPERAND2)
				{
					key_value = Key_EQUATE;
					NextOperator = OPERATOR_MULT;
				}
				else
				{	
					Operator = OPERATOR_MULT;
					key_value = Key_OPERAND2;
				}
			}	
			else if (c == Key_DIV)
			{
				if (current_operand==OPERAND2)
				{
					key_value = Key_EQUATE;
					NextOperator = OPERATOR_DIV;
				}
				else
				{	
					Operator = OPERATOR_DIV;
					key_value = Key_OPERAND2;
				}
			}	
			else if (c == Key_CHS)
			{
				if (current_operand==OPERAND1)
				{
					dOperand1 = atof(Operand1_Buffer);
					dOperand1 = dOperand1 * -1;
					FormatOperand(dOperand1, Operand1_Buffer);
				}	
				else
				{
					dOperand2 = atof(Operand2_Buffer);
					dOperand2 = dOperand2 * -1;
					FormatOperand(dOperand2, Operand2_Buffer);
				}
				WriteKeyboardCaption(Operand1_Buffer, Operator, Operand2_Buffer);
			}
			else if (c == Key_KOMMA)
			{				
				if ((CurrentChar < CALC_BUFFER_SIZE) && (comma == 0))
				{
					comma++;
					if (current_operand==OPERAND1)
					{
						Operand1_Buffer[CurrentChar++] = 46;
						Operand1_Buffer[CurrentChar] = 0;
					}
					else
					{	
						Operand2_Buffer[CurrentChar++] = 46;
						Operand2_Buffer[CurrentChar] = 0;
					} 	
				}
				WriteKeyboardCaption(Operand1_Buffer, Operator, Operand2_Buffer);
			}		
			else if (c== Key_MPLUS)
			{
				if (current_operand==OPERAND1)
					dMemory = dMemory + atof(Operand1_Buffer);
				else
					dMemory = dMemory + atof(Operand2_Buffer);
			}
			else if (c== Key_MMINUS)
			{
				if (current_operand==OPERAND1)
					dMemory = dMemory - atof(Operand1_Buffer);
				else
					dMemory = dMemory - atof(Operand2_Buffer);
			}
			else if (c== Key_MRECALL)
			{
				if (current_operand==OPERAND1)
					FormatOperand(dMemory, Operand1_Buffer);
				else
					FormatOperand(dMemory, Operand2_Buffer);
				WriteKeyboardCaption(Operand1_Buffer, Operator, Operand2_Buffer);	
			}
			else if (c== Key_MCLEAR)
			{
				dMemory = 0;
			}
			
			// Key-Value auswerten
			if (key_value < 10)
			{				
				if (CurrentChar < CALC_BUFFER_SIZE)
				{
					if (current_operand==OPERAND1)
					{
						Operand1_Buffer[CurrentChar++] = KEYBOARD_0  + key_value;
						Operand1_Buffer[CurrentChar] = 0;
					}
					else
					{	
						Operand2_Buffer[CurrentChar++] = KEYBOARD_0  + key_value;
						Operand2_Buffer[CurrentChar] = 0;
					} 	
				}
				WriteKeyboardCaption(Operand1_Buffer, Operator, Operand2_Buffer);
			}		
			else if (key_value == Key_OPERAND2)
			{
				current_operand = OPERAND2;
				comma = 0;
				CurrentChar = 0;
				WriteKeyboardCaption(Operand1_Buffer, Operator, Operand2_Buffer);
			}
			else if (key_value == Key_EQUATE)
			{
				dOperand1 = atof(Operand1_Buffer);
				dOperand2 = atof(Operand2_Buffer);
				if (Operator == OPERATOR_PLUS)
					dOperand1 = dOperand1 + dOperand2;
				else if (Operator == OPERATOR_MINUS)	
					dOperand1 = dOperand1 - dOperand2;
				else if (Operator == OPERATOR_MULT)	
					dOperand1 = dOperand1 * dOperand2;
				else if (Operator == OPERATOR_DIV)	
				{	
					if (dOperand2 == 0)
					{
						WriteKeyboardCaption(str_InputError, 32, "");
						_delay_ms(1000);
					}	
					else	
						dOperand1 = dOperand1 / dOperand2;
				}	
				// Operanden zurcksetzen
				Operator = NextOperator;
				NextOperator = OPERATOR_NULL;
				
				Operand2_Buffer[0] = 0;
				CurrentChar = 0;
				comma = 0;
				if (Operator == OPERATOR_NULL)
					current_operand = OPERAND1;
				// fValue in String wandeln
				FormatOperand(dOperand1, Operand1_Buffer);
				dOperand2 = 0;
				WriteKeyboardCaption(Operand1_Buffer, Operator, Operand2_Buffer);
			}
				
		}	 
		
	}
	
}


