/****************************************************************************
 *
 *
 * File              : grafik.h
 * Compiler          : WinAVR 4.11
 * Revision          : $Revision: 0.99 $
 * Date              : $Date: Wednesday, May 15, 2013 $
 * Updated by        : $Author: Frank Brennecke $
 *
 * Supported devices : ATmega8515
 *
 * Description       : Interface definitions for ATMega8515 based Graphics Unit
 *
 ****************************************************************************/



////////////// Grafik-Modi /////////////////////////

#define MODE_TERMINAL               0	// Terminalmodus
#define MODE_BALKENGRAFIK			1	// Balkengrafik
#define MODE_MESSWERTE              2   // Messwerte
#define MODE_MENU                   3   // Menue
#define MODE_KEYBOARD_ALPHA         4   // Bildschirmtastatur fr Texteingabe
#define MODE_KEYBOARD_NUMERIC       5   // Bildschirmtastatur fr Zahleneingabe
#define MODE_MESSAGEBOX				6 	// Auswahlfenster Ja-Nein / Ok-Abbrechen
#define MODE_8COLOR_LED				7 	// 8fach-LED-Bildschirm
#define MODE_3COLOR_LED     		8	// 3fach-LED in Ampelanordnung und -farbe
#define MODE_DRAW                   9  	// Freie Grafiken zeichnen
#define MODE_COLORPALETTE			10 	// Farbpalette - Rechtecke mit Farbe aufsteigend von 0..255
#define MODE_SLIDER					11 	// Schieberegler
#define MODE_TOUCHSCREEN_CONFIG		12	// wie MODE_DRAW, sendet aber Konfig-Info an den Touchscreen 
#define MODE_KEYBOARD_DRAW			13	// Tastatur ohne Beschriftungen
#define MODE_LAST 					MODE_KEYBOARD_DRAW


////////////// Generelle Steuerzeichen /////////////////////////

#define MODE_QUIT	31
#define MODE_END_INTERMEDIATE	30


////////////// Genutzte Farbwerte /////////////////////////
#define WindowBkColor	41
#define TextColor		255
#define DrawBkColor		0


////////////// Einzelne Grafikmodi /////////////////////////

// Terminal Mode - MODE_TERMINAL
// Text wird von links nach rechts und oben nach unten ausgegeben
// mit dem Code MODE_QUIT wird der Modus beendet
// Wenn der Bildschirm voll ist, wird er geloescht
// Weitere Steuerzeichen
#define TERMINAL_CURSORHOME             1 		// Textausgabeposition nach links oben setzen
#define TERMINAL_CLEARSCREEN            2 		// Bildschirm loeschen
#define TERMINAL_TEXTCOLOR              5		// Textfarbe als Wert (ein weiterer einzelner Wert zwischen 0 und 255)
#define TERMINAL_BKCOLOR                7		// Hintergrundfarbe als Wert (ein weiterer einzelner Wert zwischen 0 und 255)
#define TERMINAL_LINEFEEED              10		// Line Feed - Zeilenvorschub (Standard-Steuerzeichen)
#define TERMINAL_CARRIAGERETURN         13		// Carriage Return - Wagenruecklauf (Standard-Steuerzeichen)
#define TERMINAL_SETCURSOR              14		// Cursor-Position setzen (weitere 2 Werte fr x, y erforderlich)
#define TERMINAL_FONT_SMALL             16		// Fontgroesse 0 (Kleine Schrift)
#define TERMINAL_FONT_MEDIUM            17		// Fontgroesse 1 (doppelt hohe Schrift)
#define TERMINAL_FONT_BIG               18		// Fontgroesse 2 (doppelt hohe und doppelt breite Schrift)



// Bakengrafikanzeige - MODE_BALKENGRAFIK
// stellt bis zu 8 Balken auf dem Bildschirm dar, die unabhngig voneinander gendert werden knnen
// es wird keine Achsendarstellung hinzugefuegt
// Steuerzeichen
// mit dem Code 31 wird der Modus beendet
// Weitere Steuerzeichen
#define BALKENGRAFIK_NUMACHSEN_1        1   // 1 Balken zeichnen
#define BALKENGRAFIK_NUMACHSEN_2        2   // 2 Balken zeichnen
#define BALKENGRAFIK_NUMACHSEN_3        3   // 3 Balken zeichnen
#define BALKENGRAFIK_NUMACHSEN_4        4   // 4 Balken zeichnen
#define BALKENGRAFIK_NUMACHSEN_5        5   // 5 Balken zeichnen
#define BALKENGRAFIK_NUMACHSEN_6        6   // 6 Balken zeichnen
#define BALKENGRAFIK_NUMACHSEN_7        7   // 7 Balken zeichnen
#define BALKENGRAFIK_NUMACHSEN_8        8   // 8 Achsen zeichnen
#define BALKENGRAFIK_COLOR_BALKEN_1     11  // Farbe fr Balken 1 setzen - ein zustzlicher Wert zwischen 0 und 255 fr die Farbe erforderlich
#define BALKENGRAFIK_COLOR_BALKEN_2     12  // Farbe fr Balken 1 setzen - ein zustzlicher Wert zwischen 0 und 255 fr die Farbe erforderlich
#define BALKENGRAFIK_COLOR_BALKEN_3     13  // Farbe fr Balken 1 setzen - ein zustzlicher Wert zwischen 0 und 255 fr die Farbe erforderlich
#define BALKENGRAFIK_COLOR_BALKEN_4     14  // Farbe fr Balken 1 setzen - ein zustzlicher Wert zwischen 0 und 255 fr die Farbe erforderlich
#define BALKENGRAFIK_COLOR_BALKEN_5     15  // Farbe fr Balken 1 setzen - ein zustzlicher Wert zwischen 0 und 255 fr die Farbe erforderlich
#define BALKENGRAFIK_COLOR_BALKEN_6     16  // Farbe fr Balken 1 setzen - ein zustzlicher Wert zwischen 0 und 255 fr die Farbe erforderlich
#define BALKENGRAFIK_COLOR_BALKEN_7     17  // Farbe fr Balken 1 setzen - ein zustzlicher Wert zwischen 0 und 255 fr die Farbe erforderlich
#define BALKENGRAFIK_COLOR_BALKEN_8     18  // Farbe fr Balken 1 setzen - ein zustzlicher Wert zwischen 0 und 255 fr die Farbe erforderlich
#define BALKENGRAFIK_COLOR_BLUE         21  // Farbe dunkelblau fr alle Balken
#define BALKENGRAFIK_COLOR_LTBLUE       22  // Farbe dunkelblau fr alle Balken
#define BALKENGRAFIK_COLOR_GREEN        23  // Farbe dunkelgruen fr alle Balken
#define BALKENGRAFIK_COLOR_LTGREEN      24  // Farbe hellgruen fr alle Balken
#define BALKENGRAFIK_COLOR_YELLOW       25  // Farbe dunkelgelb fr alle Balken
#define BALKENGRAFIK_COLOR_LTYELLOW	    26  // Farbe hellgelb fr alle Balken
#define BALKENGRAFIK_COLOR_RED          27  // Farbe rot fr alle Balken
#define BALKENGRAFIK_COLOR_ORANGE       28  // Farbe orange fr alle Balken
#define BALKENGRAFIK_COLOR_WHITE        29  // Farbe wei fr alle Balken
#define BALKENGRAFIK_DRAWBAR_1          32   // Zeichne Balken 1  - ein zustzlicher Wert zwischen 52 und 251 fr die Hhe erforderlich
#define BALKENGRAFIK_DRAWBAR_2          33   // Zeichne Balken 1  - ein zustzlicher Wert zwischen 52 und 251 fr die Hhe erforderlich
#define BALKENGRAFIK_DRAWBAR_3          34   // Zeichne Balken 1  - ein zustzlicher Wert zwischen 52 und 251 fr die Hhe erforderlich
#define BALKENGRAFIK_DRAWBAR_4          35   // Zeichne Balken 1  - ein zustzlicher Wert zwischen 0 und 251 fr die Hhe erforderlich
#define BALKENGRAFIK_DRAWBAR_5          36   // Zeichne Balken 1  - ein zustzlicher Wert zwischen 0 und 251 fr die Hhe erforderlich
#define BALKENGRAFIK_DRAWBAR_6          37   // Zeichne Balken 1  - ein zustzlicher Wert zwischen 0 und 251 fr die Hhe erforderlich
#define BALKENGRAFIK_DRAWBAR_7          38   // Zeichne Balken 1  - ein zustzlicher Wert zwischen 0 und 251 fr die Hhe erforderlich
#define BALKENGRAFIK_DRAWBAR_8          39   // Zeichne Balken 1  - ein zustzlicher Wert zwischen 0 und 251 fr die Hhe erforderlich
#define BALKENGRAFIK_BAROFFSET			52	 // muss zu allen Balken hinzugezhlt werden




// Messwert-Modus - MODE_MESSWERTE
// fortlaufender Punkt, der von links nach rechts ausgegeben wird
// wenn der rechte Rand erreicht ist, wird links wieder angefangen
// Wertebereich: 32..255, wobei 255 oben und 32 unten ist
// mit dem Code MODE_QUIT wird der Modus beendet
// Weitere Steuerzeichen
#define MESSWERTE_RASTER			1		// zeichnet ein Raster
#define MESSWERTE_CLEARSCREEN   2 		// Bildschirm loeschen	
#define MESSWERTE_TEXTCOLOR		5		// Textfarbe als Wert (ein weiterer einzelner Wert zwischen 0 und 255)
#define MESSWERTE_START         32      // Mindestwert fr anzuzeigende Werte
#define MESSWERTE_WERTE          20     // folgende Werte>32 werdeb als Messwerte interpretiert
#define MESSWERTE_TEXT           21     // folgende Werte>32 werdeb als Text interpretiert



// Tastatur-Modus - MODE_KEYBOARD_ALPHA
// Tastatur-Modus - MODE_KEYBOARD_NUMERIC
// Es wird eine alphanumerische oder rein numerische Tastatur angezeigt
// mit dem Code MODE_QUIT wird der Modus beendet
// Weitere Steuerzeichen// zeigt eine Tastatur auf dem Touchscreen an
#define KEYBOARD_RELEASE       1     // wird in Addition mit den anderen KEYBAORD_XXX-Konstanten benutzt - Knopf wird ungedrckt gezeicnet
#define KEYBOARD_PRESSED      39     // wird in Addition mit den anderen KEYBAORD_XXX-Konstanten benutzt - Knopf wird gedrckt gezeicnet
#define KEYBOARD_SELECT       77     // Taste ist selektiert (bei Tastaturauswahl
#define KEYBOARD_TEXT        115     // Text ist besttigt und wird ausgegeben
#define KEYBOARD_SWITCHTERMINAL	253	// schaltet temporr in den Terminalmodus um
#define KEYBOARD_RELEASE31	 254	 // Button 31 wird ungedrckt dargestellt
#define KEYBOARD_RESET		 255	 // setzt Textposition auf Anfang 
#define KEYBOARD_0            48     // Tastencode fr 0 
#define KEYBOARD_A            65     // Tastencode fr A 
#define KEYBOARD_OK           36     // Tastennummer fr OK 
#define KEYBOARD_CANCEL       37     // Tastennummer fr Cancel

#define KEYBOARD_ALPHA          0   // alphanumerisches Keyboard
#define KEYBOARD_NUMERIC        1   // numerisches Keyboard
#define KEYBOARD_DRAW     		2   // Taschenrechner-Darstellung





// Menue-Modus _ MODE_MENU
// Sechszeiliges Menue - Texte sind beliebig, werden aber am Zeilenende abgeschnitten
// mit dem Code MODE_QUIT wird der Modus beendet
// Weitere Steuerzeichen
#define MENU_LINE1_SELECTED     1       // Zeile 1 selektiert
#define MENU_LINE2_SELECTED     2       // Zeile 1 selektiert
#define MENU_LINE3_SELECTED     3       // Zeile 1 selektiert
#define MENU_LINE4_SELECTED     4       // Zeile 1 selektiert
#define MENU_LINE5_SELECTED     5       // Zeile 1 selektiert
#define MENU_LINE6_SELECTED     6       // Zeile 1 selektiert
#define MENU_TEXT_LINE1         21      // Text wird ab sofort in Zeile 1 ausgegeben (berschrift, schwarz auf wei)
#define MENU_TEXT_LINE2         22      // Text wird ab sofort in Zeile 2 ausgegeben (linker Rand, wei auf blau)
#define MENU_TEXT_LINE3         23      // Text wird ab sofort in Zeile 3 ausgegeben (linker Rand)
#define MENU_TEXT_LINE4         24      // Text wird ab sofort in Zeile 4 ausgegeben (linker Rand)
#define MENU_TEXT_LINE5         25      // Text wird ab sofort in Zeile 5 ausgegeben (linker Rand)
#define MENU_TEXT_LINE6         26      // Text wird ab sofort in Zeile 6 ausgegeben (linker Rand)
#define MENU_DRAWFRAME          29      // Es werden Rahmen um die 6 Elemente ausgegeben (Standard). Dabei wird ein Rahmen selektiert dargestellt



// Menue-Modus - MODE_MESSAGEBOX
// Fenster mit  unterschiedlichen Darstellungsfunktionen
// mit dem Code MODE_QUIT wird der Modus beendet
// Weitere Steuerzeichen

#define MESSAGEBOX_BACK_NEXT		1		// Pfeile nach links und nach rechts
#define MESSAGEBOX_BACK_OK_NEXT		2		// Pfeile nach links und nach rechts, auerdem OK in der Mitte
#define MESSAGEBOX_OK_CANCEL        3		// OK und Abbrechen
#define MESSAGEBOX_YES_NO           4		// Ja und Nein
#define MESSAGEBOX_OK               5		// OK 

#define MESSAGEBOX_SELECTBTN1		11		// linker Knopf gedrckt
#define MESSAGEBOX_SELECTBTN2		12		// mittlerer Knopf gedrckt
#define MESSAGEBOX_SELECTBTN3		13		// rechter Knopf gedrckt

#define MESSAGEBOX_LINE1			21		// Text in Zeile 1 ausgeben
#define MESSAGEBOX_LINE2			22		// Text in Zeile 2 ausgeben
#define MESSAGEBOX_LINE3			23		// Text in Zeile 3 ausgeben
#define MESSAGEBOX_LINE4			24		// Text in Zeile 4 ausgeben


// Menue-Modus - MODE_8COLOR_LED
// Acht Rechtecke in whlbaren Farben (blau, rot, grn und gelb) in horizontaler Anordnung
// symbolisieren die acht roten LED's des Original-Microcontroller-Labs
// mit dem Code 31 wird der Modus beendet 
// Weitere Steuerzeichen

#define LED_8COLOR_BLUE				1		// Alle LEDs sind blau
#define LED_8COLOR_GREEN			2		// Alle LEDs sind gruen
#define LED_8COLOR_RED				3		// Alle LEDs sind rot
#define LED_8COLOR_YELLOW			4		// Alle LEDs sind gelb
#define LED_8COLOR_WHITE			5		// Alle LEDs sind wei

#define LED_8COLOR_MIXED			6		// Je 2 LEDs sind blau, gruen, gelb, rot
#define LED_8COLOR_RAINBOW			7		// Regenbogen-Farbanordnung

#define LED_NOBKCOLOR				25		// keine Hintergrundfarbe, wird durch LED_8COLOR_CLEARSCREEN aufgehoben

#define LED_8COLOR_CLEARSCREEN		8		// Lscht den Bildschirm

#define LED_1_OFF					9
#define LED_2_OFF					10
#define LED_3_OFF					11
#define LED_4_OFF					12
#define LED_5_OFF					13
#define LED_6_OFF					14
#define LED_7_OFF					15
#define LED_8_OFF					16
#define LED_1_ON					17
#define LED_2_ON					18
#define LED_3_ON					19
#define LED_4_ON					20
#define LED_5_ON					21
#define LED_6_ON					22
#define LED_7_ON					23
#define LED_8_ON					24



// Ampel-Mode - MODE_3COLOR_LED
// Drei Rechtecke in den Farben rot, grn und gelb in Ampelanordnung
// Symbolisieren die drei LED's des Original-Microcontroller-Labs
// mit dem Code MODE_QUIT wird der Modus beendet
// Weitere Steuerzeichen
#define LED_3COLOR_GREEN_ON    2 // gruen hell
#define LED_3COLOR_GREEN_OFF   3 // gruen dunkel
#define LED_3COLOR_YELLOW_ON   4 // gelb hell
#define LED_3COLOR_YELLOW_OFF  5 // gelb dunkel
#define LED_3COLOR_RED_ON      6 // rot hell
#define LED_3COLOR_RED_OFF     7 // rot dunkel


// Freier Zeichenmodus - MODE_DRAW
// mit dem Code 31 wird der Modus beendet
// Weitere Steuerzeichen
#define DRAW_CLEARSCREEN        2 		// Bildschirm loeschen
#define DRAW_COLOR              5		// Zeichenfarbe als Wert (ein weiterer einzelner Wert zwischen 0 und 255)
#define DRAW_BKCOLOR            7		// Hintergrundfarbe als Wert (ein weiterer einzelner Wert zwischen 0 und 255)
                                        // lscht nicht automatisch den Bildschirm!
#define DRAW_SELECTCOLOR		8		// Ruft eine Farbauswahl auf - Farbe muss vom aufrufenden Programm gesetzt werden!									
#define DRAW_LINE               14		// Linie, erfordert vier weitere Werte fr x1/y1 und x2/y2
#define DRAW_SETPIXEL           15 		// Pixel, erfordert zwei weitere Werte fr x/y
#define DRAW_DRAWRECT           19		// Rechteck, erfordert vier weitere Werte fr x1/y1 und x2/y2
#define DRAW_FILLRECT           20		// Geflltes Rechteck, erfordert vier weitere Werte fr x1/y1 und x2/y2
#define DRAW_BUTTONFFRAME       21		// zeichnet einen transparenten Knopf (links oben: hell, rechts unten: dunkel
#define DRAW_SELBUTTONFFRAME    22		// zeichnet einen selektierten Knopf (links oben: dunkel, rechts unten: hell
#define DRAW_SWITCHTERMINAL     23      // schaltet temporr in den Terminalmodus, damit Texte ausgegeben werden knnen



// Schieberegler - MODE_SLIDER
// mit dem Code MODE_QUIT wird der Modus beendet	
// Werte ber 31 werden als Text interpretiert und zweizeilig oben ausgegeben
// Weitere Steuerzeichen
#define SLIDER_REDRAW				1	// Neuzeichnen im unselektierten Zustand
#define SLIDER_BTN_BACK_PRESSED		2	// Back-Button gedrckt
#define SLIDER_BTN_NEXT_PRESSED		3	// Next-Button gedrckt
#define SLIDER_BTN_OK_PRESSED		4	// OK-Button gedrckt
#define SLIDER_VALUE				5	// Wert fr Slider - zweiter Wert wird abgefragt, zulssig sind Werte von 0 bis 200
#define SLIDER_CLEARSCREEN			6	// lscht den Bildschirm inklusive des Textes



// Touchscreen-Konfiguration - MODE_TOUCHSCREEN_CONFIG
// mit dem Code MODE_QUIT wird der Modus beendet	
// Weitere Steuerzeichen
#define TOUCHSCREEN_CONFIG_LEFT_TOP				1	// zeichnet links oben eine rote Ecke
#define TOUCHSCREEN_CONFIG_RIGHT_TOP			2	// zeichnet links oben eine grne und rechts oben eine rote Ecke
#define TOUCHSCREEN_CONFIG_LEFT_BOTTOM			3	// zeichnet rechts oben eine grne und links unten eine rote Ecke
#define TOUCHSCREEN_CONFIG_RIGHT_BOTTOM			4	// zeichnet links unten eine grne und rechts unten eine rote Ecke
#define TOUCHSCREEN_CONFIG_OK					5	// zeichnet rechts unten eine grne Ecke
#define TOUCHSCREEN_CONFIG_TERMINAL				6	// schaltet in den temporren Terminalmodus