
/****************************************************************************
 *
 *
 * File              : main.c
 * Compiler          : WinAVR 4.11
 * Revision          : $Revision: 0.99 $
 * Date              : $Date: Wednesday, May 15, 2013 $
 * Updated by        : $Author: Benedikt - Mikrocontrollerforum $
 *                     $Author2: Frank Brennecke $
 *
 * Supported devices : ATmega8515
 *
 * Description       : Graphics Controller Unit with High-Level functions
 *
 ****************************************************************************/



#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include "global.h"
#include "tv.h"
#include "uart.h"
#include "grafik.h"



#define XTAL_CPU			16000000
#define UART_BAUD_RATE		   19200 

#define F_CPU			16000000

#define TouchScreen_SendFields PORTD &= ~(1<<PD5);PORTD |= (1<<PD3)
#define TouchScreen_TurnOff PORTD &= ~(1<<PD5);PORTD &= ~(1<<PD3)
#define TouchScreen_Configure PORTD |= (1<<PD5);PORTD |= (1<<PD3)
#define TouchScreen_SendCoordinates PORTD &= ~(1<<PD3);PORTD |= (1<<PD5)

#define Mem_SetWindow0 PORTB &= ~(1<<PB0);PORTB &= ~(1<<PB1)	// Allgemein
#define Mem_SetWindow1 PORTB &= ~(1<<PB0);PORTB |= (1<<PB1)		// Grafik
#define Mem_SetWindow2 PORTB &= ~(1<<PB0);PORTB |= (1<<PB1)		// Messagebox
#define Mem_SetWindow3 PORTB |= (1<<PB0);PORTB |= (1<<PB1)		// Eingabeelemente




       
// Der Terminalmodus kann von anderen Modi aius aufgerufen werden und zur freien Textausgabe genutzt werden
// in diesem Fall wird der Bildschirm beim Betreten des Modus nicht gelscht
#define TERMINAL_INTERMEDIATE   1

// Stringkonstanten
const char strOK[]="OK";
const char strCancel[]="Abbrechen";
const char strYes[]="Ja";
const char strNo[]="Nein";
const char strBack[]="<<";
const char strNext[]=">>";

void WriteString(unsigned char x, unsigned char y, unsigned char textfarbe, unsigned int hintergrundfarbe, const char *s)
{
    while (*s) 
	{
		tv_writechar2x(x, y, textfarbe, hintergrundfarbe, *s++);
		x+=Pixelwidth;
	}
}


void LoescheBildschirm(unsigned char farbe)
{
	tv_clear(farbe);
	tv_line (0,0,0,YSIZE-1,farbe);
	tv_line (0,0,XSIZE-1,0,farbe);
	tv_line (XSIZE-1,0,XSIZE-1,YSIZE-1,farbe);
	tv_line (0,YSIZE-1,XSIZE-1,YSIZE-1,farbe);
}


void DrawButton(unsigned char x1,unsigned char y1,unsigned char x2,unsigned char y2,unsigned char selected)
{
	unsigned char i;
	for (i=x1; i<=x2; i++)
	{
		tv_setpixel (i,y1,255-selected);
		tv_setpixel (i,y2,selected);
	}	
	for (i=y1; i<=y2; i++)
	{
		tv_setpixel (x1,i,255-selected);
		tv_setpixel (x2,i,selected);
	}	
}





void Terminal(unsigned char IntermediateMode)
{
    // Terminal Mode - MODE_TERMINAL
    // Text wird von links nach rechts und oben nach unten ausgegeben
    // mit dem Code MODE_QUIT wird der Modus beendet
    // Wenn der Bildschirm voll ist, wird er geloescht
    // Weitere Steuerzeichen
    // #define TERMINAL_CURSORHOME      1 		// Textausgabeposition nach links oben setzen
    // #define TERMINAL_CLEARSCREEN     2 		// Bildschirm loeschen
    // #define TERMINAL_TEXCOLOR_RGB 	4 		// Textfarbe als RGB (weitere 3 Werte fr rot, gruen, blau erforderlich)
    // #define TERMINAL_TEXTCOLOR		5		// Textfarbe als Wert (ein weiterer einzelner Wert zwischen 0 und 255)
    // #define TERMINAL_BKCOLOR_RGB     6		// Hintergrundfarbe als RGB (weitere 3 Werte fuer rot, gruen, blau erforderlich)
    // #define TERMINAL_BKCOLOR         7		// Hintergrundfarbe als Wert (ein weiterer einzelner Wert zwischen 0 und 255)
    // #define TERMINAL_BACKSPACE		8		// Rueckschritttaste - loescht das letzte Zeichen (Standard-Steuerzeichen)
    // #define TERMINAL_LINEFEEED		10		// Line Feed - Zeilenvorschub (Standard-Steuerzeichen)
    // #define TERMINAL_CLEARPOSITION	11		// Loescht das Zeichen an der Textausgabeposition durch Ausgabe eine Leerzeichens
    // #define TERMINAL_FORMFEED		12		// Form Feed - Seitenvorschub, loescht den Bildschirm und setzt den Cursor auf links oben (Standard-Steuerzeichen)
    // #define TERMINAL_CARRIAGERETURN	13		// Carriage Return - Wagenruecklauf (Standard-Steuerzeichen)
    // #define TERMINAL_SETCURSOR		14		// Cursor-Position setzen (weitere 2 Werte fr x, y erforderlich)
    // #define TERMINAL_FONT_SMALL		16		// Fontgroesse 0 (Kleine Schrift)
    // #define TERMINAL_FONT_MEDIUM     17		// Fontgroesse 1 (doppelt hohe Schrift)
    // #define TERMINAL_FONT_BIG		18		// Fontgroesse 2 (doppelt hohe und doppelt breite Schrift)

	unsigned char c;
	unsigned char textbg=0; // schwarz
	unsigned char xpos=0, ypos=0,textfg=TextColor; // wei
	unsigned char fontsize = 1; // Standardfont festlegen


	
// Speicherseite whlen	- bisher nicht implementiert
	
	if (IntermediateMode != TERMINAL_INTERMEDIATE)    // Bildschirm nur loeschen, wenn der Terminalmodus von auen eingestellt wurde
	{	
		LoescheBildschirm(0);
		// Touchscreen abschalten
		TouchScreen_TurnOff;
	}	
	for (;;)
	{	
		c = uart_getchar();
		if (c > MODE_QUIT)				//Display Text
		{	
			if (ypos>=236)
			{
				ypos=0;
				LoescheBildschirm(0);
			}	
			if (fontsize == 0)
			{
				tv_writechar(xpos,ypos,textfg,textbg,c);
				xpos+=Pixelwidth;
				if (xpos>250)
				{	
					xpos=0;
					ypos+=8;
				}
			}
			else if (fontsize == 1)
			{
				tv_writechar2x(xpos,ypos,textfg,textbg,c);
				xpos+=Pixelwidth;
				if (xpos>250)
				{	
					xpos=0;
					ypos+=16;
					
				}
			}
			else
			{
				tv_writechar3x(xpos,ypos,textfg,textbg,c);
				xpos+=2*Pixelwidth;
				if (xpos>236)
				{	
					xpos=0;
					ypos+=24;
					
				}
			}
		}
	
		else if (c==TERMINAL_CURSORHOME)			//Cursor Home
		{	
			xpos=0;
			ypos=0;
		}
		else if (c==TERMINAL_CLEARSCREEN)
		{
			LoescheBildschirm(0);
		}
		else if (c==TERMINAL_TEXTCOLOR)			//Set Foreground Color (Color Value)
		{	
			textfg=uart_getchar();
		}
		else if (c==TERMINAL_BKCOLOR)			//Set Background Color (Color Value)
		{	
			textbg=uart_getchar();	
		}
		/*
		else if (c==TERMINAL_BACKSPACE)			//Backspace  
		{	
// arbeitet nur mit kleiner Schrift korrekt					
			xpos-=Pixelwidth;
			if (xpos>246)
			{	xpos=246;
				ypos-= (8*(1+fontsize));
				if (ypos>240)
					ypos=240;				
			}	
			tv_writechar(xpos,ypos,textfg,textbg,32);
		}
		*/
		else if (c==TERMINAL_LINEFEEED)        //Line Feed
		{	
			ypos+=(8*(1+fontsize));
			if (ypos>=248) 
				ypos=0;
		}
		/*
		else if (c==TERMINAL_CLEARPOSITION)        //Delete in place
		{	
// arbeitet nur mit kleiner Schrift korrekt					
			tv_writechar(xpos,ypos,textfg,textbg,32);
		}
		
		else if (c==TERMINAL_FORMFEED)        //Form Feed
		{	
			unsigned int i;
			unsigned char *mem=(unsigned char*)pixel;
			xpos=0;
			ypos=0;
			for (i=0;i<((unsigned int)XSIZE*YSIZE); i++)
				*mem++=0;
		}
		*/
		else if (c==TERMINAL_CARRIAGERETURN)			//Carriage Return
		{
			xpos=0;
		}
		else if (c==TERMINAL_SETCURSOR)        //Set Cursor Position (Column and Row)
		{	
			xpos=uart_getchar();
			ypos=uart_getchar();
			if (ypos>=YSIZE)
				ypos=0;
		}
		else if (c==TERMINAL_FONT_SMALL)
		{
			fontsize = 0;  // 0 fr normale Schrift
		}
		else if (c==TERMINAL_FONT_MEDIUM)
		{
			fontsize = 1;  // 1 fr doppelte Schriftgre
		}
		else if (c==TERMINAL_FONT_BIG)
		{
			fontsize = 2;  // 2 fr doppelte Schriftgre und hheren Font
		}
		else if ((c==MODE_QUIT) || (c==MODE_END_INTERMEDIATE))
		{
			break; 			// Neuer Modus angekndigt, Terminal verlassen
		}
	}
}


void Balkengrafik(void)
{
    // Bakengrafikanzeige - MODE_BALKENGRAFIK
    // stellt bis zu 8 Balken auf dem Bildschirm dar, die unabhngig voneinander gendert werden knnen
    // es wird keine Achsendarstellung hinzugefuegt
    // Steuerzeichen
    // mit dem Code 31 wird der Modus beendet
    // Weitere Steuerzeichen
    // #define BALKENGRAFIK_NUMACHSEN_1    1   // 1 Balken zeichnen
    // #define BALKENGRAFIK_NUMACHSEN_2    2   // 2 Balken zeichnen
    // #define BALKENGRAFIK_NUMACHSEN_3    3   // 3 Balken zeichnen
    // #define BALKENGRAFIK_NUMACHSEN_4    4   // 4 Balken zeichnen
    // #define BALKENGRAFIK_NUMACHSEN_5    5   // 5 Balken zeichnen
    // #define BALKENGRAFIK_NUMACHSEN_6    6   // 6 Balken zeichnen
    // #define BALKENGRAFIK_NUMACHSEN_7    7   // 7 Balken zeichnen
    // #define BALKENGRAFIK_NUMACHSEN_8    8   // 8 Achsen zeichnen
    // #define BALKENGRAFIK_COLOR_BALKEN_1     11  // Farbe fr Balken 1 setzen - ein zustzlicher Wert zwischen 0 und 255 fr die Farbe erforderlich
    // #define BALKENGRAFIK_COLOR_BALKEN_2     12  // Farbe fr Balken 1 setzen - ein zustzlicher Wert zwischen 0 und 255 fr die Farbe erforderlich
    // #define BALKENGRAFIK_COLOR_BALKEN_3     13  // Farbe fr Balken 1 setzen - ein zustzlicher Wert zwischen 0 und 255 fr die Farbe erforderlich
    // #define BALKENGRAFIK_COLOR_BALKEN_4     14  // Farbe fr Balken 1 setzen - ein zustzlicher Wert zwischen 0 und 255 fr die Farbe erforderlich
    // #define BALKENGRAFIK_COLOR_BALKEN_5     15  // Farbe fr Balken 1 setzen - ein zustzlicher Wert zwischen 0 und 255 fr die Farbe erforderlich
    // #define BALKENGRAFIK_COLOR_BALKEN_6     16  // Farbe fr Balken 1 setzen - ein zustzlicher Wert zwischen 0 und 255 fr die Farbe erforderlich
    // #define BALKENGRAFIK_COLOR_BALKEN_7     17  // Farbe fr Balken 1 setzen - ein zustzlicher Wert zwischen 0 und 255 fr die Farbe erforderlich
    // #define BALKENGRAFIK_COLOR_BALKEN_8     18  // Farbe fr Balken 1 setzen - ein zustzlicher Wert zwischen 0 und 255 fr die Farbe erforderlich
    // #define BALKENGRAFIK_COLOR_BLUE         21  // Farbe dunkelblau fr alle Balken
    // #define BALKENGRAFIK_COLOR_LTBLUE       22  // Farbe dunkelblau fr alle Balken
    // #define BALKENGRAFIK_COLOR_GREEN        23  // Farbe dunkelgruen fr alle Balken
    // #define BALKENGRAFIK_COLOR_LTGREEN      24  // Farbe hellgruen fr alle Balken
    // #define BALKENGRAFIK_COLOR_YELLOW       25  // Farbe dunkelgelb fr alle Balken
    // #define BALKENGRAFIK_COLOR_LTYELLLOW    26  // Farbe hellgelb fr alle Balken
    // #define BALKENGRAFIK_COLOR_RED          27  // Farbe rot fr alle Balken
    // #define BALKENGRAFIK_COLOR_ORANGE       28  // Farbe orange fr alle Balken
    // #define BALKENGRAFIK_COLOR_WHITE        29  // Farbe wei fr alle Balken
    // #define BALKENGRAFIK_DRAWBAR_1          32   // Zeichne Balken 1  - ein zustzlicher Wert zwischen 0 und 251 fr die Hhe erforderlich
    // #define BALKENGRAFIK_DRAWBAR_2          33   // Zeichne Balken 1  - ein zustzlicher Wert zwischen 0 und 251 fr die Hhe erforderlich
    // #define BALKENGRAFIK_DRAWBAR_3          34   // Zeichne Balken 1  - ein zustzlicher Wert zwischen 0 und 251 fr die Hhe erforderlich
    // #define BALKENGRAFIK_DRAWBAR_4          35   // Zeichne Balken 1  - ein zustzlicher Wert zwischen 0 und 251 fr die Hhe erforderlich
    // #define BALKENGRAFIK_DRAWBAR_5          36   // Zeichne Balken 1  - ein zustzlicher Wert zwischen 0 und 251 fr die Hhe erforderlich
    // #define BALKENGRAFIK_DRAWBAR_6          37   // Zeichne Balken 1  - ein zustzlicher Wert zwischen 0 und 251 fr die Hhe erforderlich
    // #define BALKENGRAFIK_DRAWBAR_7          38   // Zeichne Balken 1  - ein zustzlicher Wert zwischen 0 und 251 fr die Hhe erforderlich
    // #define BALKENGRAFIK_DRAWBAR_8          39   // Zeichne Balken 1  - ein zustzlicher Wert zwischen 0 und 251 fr die Hhe erforderlich


#define ACHSENBREITE 10

	unsigned char i, c, value, AnzahlAchsen=8, x_delta=32, x_start=11, xpos, x, y;
	unsigned char graf[8]={1, 31, 40, 56, 144, 254, 128, 176};
        // blau, hellblau, dunkelgruen, hellgruen, dunkelgelb, hellgelb, rot, orange
	
// Touchscreen abschalten
	TouchScreen_TurnOff;
    
	LoescheBildschirm(0);
	
	for (;;)
	{	
		c = uart_getchar();
		if (c==MODE_QUIT)
			break;
		if ((c >= BALKENGRAFIK_NUMACHSEN_1) && (c <= BALKENGRAFIK_NUMACHSEN_8))
		{
			if (c != AnzahlAchsen)
			{
				if ((c > 0) && (c < 9))
				{
					AnzahlAchsen = c;
					x_delta = (256 - (AnzahlAchsen * ACHSENBREITE)) / AnzahlAchsen + ACHSENBREITE;
					x_start = (x_delta-ACHSENBREITE) / 2;
					LoescheBildschirm(0);	
				}	
			}	
		}
		else if ((c >= BALKENGRAFIK_COLOR_BALKEN_1) && (c <= BALKENGRAFIK_COLOR_BALKEN_8))
		{
			graf[c-BALKENGRAFIK_COLOR_BALKEN_1] = uart_getchar();  	// Farbe setzen
		}
		else if ((c >= BALKENGRAFIK_COLOR_BLUE) && (c <= BALKENGRAFIK_COLOR_WHITE))		// einheitliche Farbe fr alle Balken setzen
		{
			for (i=0; i<=7; i++)
			{
				if (c == BALKENGRAFIK_COLOR_BLUE)
					graf[i] = 1;  	// blau
				else if (c == BALKENGRAFIK_COLOR_LTBLUE)
					graf[i] = 31;  	// hellblau
				else if (c == BALKENGRAFIK_COLOR_GREEN)
					graf[i] = 40; 	// dunkelgruen
				else if (c == BALKENGRAFIK_COLOR_LTGREEN)
					graf[i] = 56;  	// hellgruen
				else if (c == BALKENGRAFIK_COLOR_YELLOW)
					graf[i] = 144; 	// dunkelgelb
				else if (c == BALKENGRAFIK_COLOR_LTYELLOW)
					graf[i] = 254; 	// hellgelb
				else if (c == BALKENGRAFIK_COLOR_RED)
					graf[i] = 128; 	// rot	
				else if (c == BALKENGRAFIK_COLOR_ORANGE)
					graf[i] = 176;  // orange
				else if (c == BALKENGRAFIK_COLOR_WHITE)
					graf[i] = 255;  //wei		
			}
		}
		else if ((c >= BALKENGRAFIK_DRAWBAR_1) && (c <= BALKENGRAFIK_DRAWBAR_8))	// Balken zeichnen
		{
			c-=BALKENGRAFIK_DRAWBAR_1;			// Wert zwischen 0..7
			xpos = x_start + c * x_delta;
			
			value = uart_getchar();				// Hhe des Balkens
			if (value >= BALKENGRAFIK_BAROFFSET)
			{
				value-=BALKENGRAFIK_BAROFFSET;		// ein Wert zwischen 0 und 200
				
				value = YSIZE - value - 25;			// Ein Wert zwischen 226 und 26	
				
				for (y=26; y<=value; y++)
					for (x=xpos; x<=xpos+ACHSENBREITE; x++)
						pixel[y][x]=0;
				for (y=value; y<YSIZE-26; y++)
					for (x=xpos; x<=xpos+ACHSENBREITE; x++)
						pixel[y][x]=graf[c];
			}			
		}
	}	
}

//unsigned char Messwerte(unsigned char lastpos)
void Messwerte(void)
{
    // Messwert-Modus - MODE_MESSWERTE
    // fortlaufender Punkt, der von links nach rechts ausgegeben wird
    // wenn der rechte Rand erreicht ist, wird links wieder angefangen
    // Wertebereich: 32..251, wobei 251 oben und 32 unten ist
    // mit dem Code MODE_QUIT wird der Modus beendet
    // Weitere Steuerzeichen
	// #define MESSWERTE_RASTER			1		// zeichnet ein Raster
    // #define MESSWERTE_CLEARSCREEN   2 		// Bildschirm loeschen
    // #define MESSWERTE_TEXTCOLOR		5		// Textfarbe als Wert (ein weiterer einzelner Wert zwischen 0 und 255)
    // #define MESSWERTE_START         32      // Mindestwert fr anzuzeigende Werte
    // #define MESSWERTE_WERTE           20     // folgende Werte>32 werdeb als Messwerte interpretiert
    // #define MESSWERTE_TEXT           21     // folgende Werte>32 werdeb als Text interpretiert  
    
    
	unsigned char c=MESSWERTE_RASTER;
	unsigned char xpos=0, textpos=16, drawmode=0, graf=TextColor, MemoryPage;
// Touchscreen abschalten
    TouchScreen_TurnOff;
	
	MemoryPage = PORTB;
	Mem_SetWindow1;
/*	
	xpos = lastpos;
    if (xpos == 255)
	{
		LoescheBildschirm(0);
		xpos = 0;
	}	
*/	
	LoescheBildschirm(0);
	for (;;)
	{	
		
		if (c==MESSWERTE_RASTER)  // zeichnet Raster
		{
			unsigned char i,j;
			for (i=0;i<10;i++)	
				for (j=0;j < 22;j++)
				{
					tv_setpixel(i*25, j*10, 41);	
					tv_setpixel(i*25, j*10+1, 41);
				}	
			for (j=0;j < 5;j++)	
				for (i=0;i<50;i++)
				{
					tv_setpixel(i*5, j*50+20, 41);	
					tv_setpixel(i*5, j*50+21, 41);
				}		
			c=0;		
				
		}
		if (c==MESSWERTE_CLEARSCREEN)
		{
			xpos = 0;
            textpos = 16;
			LoescheBildschirm(0);
			c=MESSWERTE_RASTER;	// Raster neu zeichnen
		}
		else if (c==MESSWERTE_TEXTCOLOR)
		{
			graf = uart_getchar();
			if (graf == 0)		// Schwarz als Farbe nicht zugelassen 
				graf = 255;
		}
		else if (c==MESSWERTE_WERTE)
            drawmode = 0;
        else if (c==MESSWERTE_TEXT)
            drawmode = 1; 
		else if (c > MODE_QUIT)
		{
			if (drawmode==0)
            {
                tv_line(xpos,0,xpos,YSIZE-32,0);  // aktuelle Spalte loeschen
				if (c <= YSIZE) 
				{
					tv_setpixel(xpos, YSIZE-c, graf);   		// Datenpixel ausgeben
					tv_setpixel(xpos++, YSIZE-c+1, graf);   	// Datenpixel erneut ausgeben
				}	
				if (xpos == 254)
				{
                    xpos = 0;	// am Anfang neu beginnen
					c=MESSWERTE_RASTER;
				}	
            }
            else
            {
                tv_writechar2x(textpos,232,graf,0,c);
				textpos+=Pixelwidth;
            }
		}
		if (c != MESSWERTE_RASTER)
			c = uart_getchar();
		if (c==MODE_QUIT)
			break;
	}
	PORTB = MemoryPage;
//	return xpos;
}


#define SELECTEDCOLOR		18


void DrawKeyboardButton(unsigned char c, unsigned char Color)
{
	unsigned char i, j;
	if (c==KEYBOARD_OK)
		DrawButton(128,210,190,251, Color);
	else if (c==KEYBOARD_CANCEL)
		DrawButton(192,210,254,250, Color);		
	else
	{
		i = c/8;
		j = c - i*8;
		DrawButton(j*32, i*42+42, j*32+30, i*42+82, Color);
	}	
}

void Texteingabe(unsigned char Keyboard_Mode)
{
    // Tastatur-Modus - MODE_KEYBOARD_ALPHA
    // Tastatur-Modus - MODE_KEYBOARD_NUMERIC
    // Es wird eine alphanumerische oder rein numerische Tastatur angezeigt
    // mit dem Code MODE_QUIT wird der Modus beendet
    // Weitere Steuerzeichen// zeigt eine Tastatur auf dem Touchscreen an
    // #define KEYBOARD_RELEASE       1     // wird in Addition mit den anderen KEYBAORD_XXX-Konstanten benutzt - Knopf wird ungedrckt gezeicnet
    // #define KEYBOARD_PRESSED      37     // wird in Addition mit den anderen KEYBAORD_XXX-Konstanten benutzt - Knopf wird gedrckt gezeicnet
    // #define KEYBOARD_SELECT       73     // Taste ist selektiert (bei Tastaturauswahl
    // #define KEYBOARD_TEXT        109     // Text ist besttigt und wird ausgegeben
    // #define KEYBOARD_RELEASE31	 254	 // Button 31 wird ungedrckt dargestellt
    // #define KEYBOARD_RESET		 255	 // setzt Textposition auf Anfang 
    // #define KEYBOARD_0            48     // Tastencode fr 0 
    // #define KEYBOARD_A            65     // Tastencode fr A 

    // #define KEYBOARD_ALPHA          0   // alphanumerisches Keyboard
    // #define KEYBOARD_NUMERIC        1   // numerisches Keyboard
    // #define KEYBOARD_DRAW     		2   // Keine Tastenbeschriftungen
    
    unsigned char c, textpos=16, MemoryPage;
	unsigned char i, j;
    
//	Information an Touchscreen senden - Felder werden zurueckgegeben
    TouchScreen_SendFields;
	
	MemoryPage = PORTB;
	Mem_SetWindow3;
    
    LoescheBildschirm(WindowBkColor);
	
	
	
	i=255;
	j=42;
	while (j--)
		while (i--)
			pixel[j][i]=255;
			
    
// es wird eine Tastatur mit 8x5=40 Feldern erzeugt
// die letzten beiden Tasten sind doppelt gro 
    
    for (i=0;i<5;i++)
    {
        for (j=0;j<8;j++)
        {
            c = i*8+j;
            if (c < 36) // 36 Tasten, nullbasiert
            {
                DrawButton(j*32, i*42+42, j*32+30, i*42+82, 0);
				if (Keyboard_Mode != KEYBOARD_DRAW)
				{
					if (c<10)
					{
						c+=KEYBOARD_0;  // numerische Zeichen
						tv_writechar2x(j*32+12, i*42+57, TextColor, WindowBkColor,c);
					}
					else if (Keyboard_Mode == KEYBOARD_ALPHA)    // Alphanumerische Texte nur bei Bedarf ausgeben
					{
						c+=KEYBOARD_A-10; // Zeichen ab 65 (A,B,C,D,...)
						tv_writechar2x(j*32+12, i*42+57, TextColor, WindowBkColor,c);
					}
				}	
            }
        }
    }
// OK-Knopf zeichnen
    DrawButton(128,210,190,251,0);
    WriteString(154, 224, TextColor, WindowBkColor, strOK);
    DrawButton(192,210,254,250,0);
    WriteString(196, 224, TextColor, WindowBkColor, strCancel);
    
    c=0;
    
    for (;;)
    {
        c = uart_getchar();
		if (c==MODE_QUIT)			//
			break;
		else if (c==KEYBOARD_RESET)
			textpos=16;
		else if (c==KEYBOARD_RELEASE31)
			DrawButton(192, 168, 222, 208, 0);		// Sonderbehandlung Button 31, damit MODE_QUIT funktioniert
		else if (c==KEYBOARD_SWITCHTERMINAL)
			Terminal(TERMINAL_INTERMEDIATE);	
		else if (c >= KEYBOARD_TEXT)	// 115..253
        {
			if (textpos < 240)
            {
                tv_writechar2x(textpos, 16, 0, 255,c-KEYBOARD_TEXT);
                textpos+=Pixelwidth;
            }
        }	
        else if (c >= KEYBOARD_SELECT)	// 77..114
 			DrawKeyboardButton(c-KEYBOARD_SELECT, SELECTEDCOLOR);
        else if (c >= KEYBOARD_PRESSED)	// 39..76
			DrawKeyboardButton(c-KEYBOARD_PRESSED, 255);
        else if (c >= KEYBOARD_RELEASE) // 1..38
            DrawKeyboardButton(c-KEYBOARD_RELEASE, 0);
						
    }
	PORTB = MemoryPage;
}

void Menu(void)
{
    // Menue-Modus _ MODE_MENU
    // Sechszeiliges Menue - Texte sind beliebig, werden aber am Zeilenende abgeschnitten
    // mit dem Code MODE_QUIT wird der Modus beendet
    // Weitere Steuerzeichen
    // #define MENU_LINE1_SELECTED     1       // Zeile 1 selektiert
    // #define MENU_LINE2_SELECTED     2       // Zeile 1 selektiert
    // #define MENU_LINE3_SELECTED     3       // Zeile 1 selektiert
    // #define MENU_LINE4_SELECTED     4       // Zeile 1 selektiert
    // #define MENU_LINE5_SELECTED     5       // Zeile 1 selektiert
    // #define MENU_LINE6_SELECTED     6       // Zeile 1 selektiert
    // #define MENU_TEXT_LINE1         21      // Text wird ab sofort in Zeile 1 ausgegeben (linker Rand)
    // #define MENU_TEXT_LINE2         22      // Text wird ab sofort in Zeile 1 ausgegeben (linker Rand)
    // #define MENU_TEXT_LINE3         23      // Text wird ab sofort in Zeile 1 ausgegeben (linker Rand)
    // #define MENU_TEXT_LINE4         24      // Text wird ab sofort in Zeile 1 ausgegeben (linker Rand)
    // #define MENU_TEXT_LINE5         25      // Text wird ab sofort in Zeile 1 ausgegeben (linker Rand)
    // #define MENU_TEXT_LINE6         26      // Text wird ab sofort in Zeile 1 ausgegeben (linker Rand)
    // #define MENU_DRAWFRAME          30      // Es werden Rahmen um die 6 Elemente ausgegeben (Standard). Dabei wird ein Rahmen selektiert dargestellt
    
	unsigned char c=30, i;
	unsigned char xpos=16, ypos=10, x, y, selected=1, bkgraf=WindowBkColor, graf=TextColor, MemoryPage;
	
	MemoryPage = PORTB;
	Mem_SetWindow2;
	
	LoescheBildschirm(WindowBkColor);

	x=255;
	y=42;
	while (y--)
		while (x--)
			pixel[y][x]=255;		
	
    
//	Information an Touchscreen senden - Felder werden zurckgegeben
    TouchScreen_SendFields;
	
	for (;;)
	{	
		if (c > MODE_QUIT) // Text ausgeben
		{
			if (xpos < 240)
			{
				tv_writechar2x(xpos, ypos, graf, bkgraf, c);
				xpos+=Pixelwidth;
			}	
		}
		else if ((c>=MENU_LINE1_SELECTED) && (c<=MENU_LINE6_SELECTED))	// Selektion setzen
		{
			
			selected = c-1;
			c=30;
		}
		else if ((c>=MENU_TEXT_LINE1) && (c<=MENU_TEXT_LINE6))	// Zeile fr Textausgabe festlegen
		{
			ypos = (c-21)*42 + 12;
			xpos = 16;
			
			if (c==MENU_TEXT_LINE1)
			{
				graf = 0;
				bkgraf = 255;
			}	
			else
			{
				graf = TextColor;
				bkgraf = WindowBkColor;
			}	
			
		}
		if (c==MENU_DRAWFRAME)	// Rahmen zeichnen
		{
			for (i=1;i<6;i++)
			{
				y = i * 42 + 1;
				if (i==selected)
				{
					graf = 0;
					bkgraf = 255;
				}
				else
				{
					graf = 1;
					bkgraf = WindowBkColor;
				}
				tv_line (0,y,255,y,bkgraf);
				tv_line (255,y,255,y+40,graf);
				tv_line (255,y+40,0,y+40,graf);
				tv_line (0,y+40,0,y,bkgraf);
			}
		}
		c = uart_getchar();
		if (c==MODE_QUIT)
			break; 			// Neuer Modus angekndigt, Menu verlassen
	}	
	PORTB = MemoryPage;
}


void Kalibrierung(void)
{
	// Touchscreen-Konfiguration - MODE_TOUCHSCREEN_CONFIG
	// mit dem Code MODE_QUIT wird der Modus beendet	
	// Weitere Steuerzeichen
	// #define TOUCHSCREEN_CONFIG_LEFT_TOP				1	// zeichnet links oben eine rote Ecke
	// #define TOUCHSCREEN_CONFIG_RIGHT_TOP			2	// zeichnet links oben eine grne und rechts oben eine rote Ecke
	// #define TOUCHSCREEN_CONFIG_LEFT_BOTTOM			3	// zeichnet rechts oben eine grne und links unten eine rote Ecke
	// #define TOUCHSCREEN_CONFIG_RIGHT_BOTTOM			4	// zeichnet links unten eine grne und rechts unten eine rote Ecke
	// #define TOUCHSCREEN_CONFIG_OK					5	// zeichnet rechts unten eine grne Ecke
	// #define TOUCHSCREEN_CONFIG_TERMINAL				6	// schaltet in den temporren Terminalmodus
	unsigned char c;
	
//	Information an Touchscreen senden, Kalibrierungsmodus aktiviert
	TouchScreen_Configure;		
	LoescheBildschirm(0);
	for (;;)
	{
		c = uart_getchar();
		if (c==31)
		{
			break;
		}
		else if (c==TOUCHSCREEN_CONFIG_LEFT_TOP)
		{
			LoescheBildschirm(0);
			tv_line (0,0,0,20,128);
			tv_line (0,0,10,0,128);	
		}
		else if (c==TOUCHSCREEN_CONFIG_RIGHT_TOP)
		{
			tv_line (0,0,0,20,16);
			tv_line (0,0,10,0,16);
			tv_line (255,0,255,20,128);
			tv_line (245,0,255,0,128);	
		}
		else if (c==TOUCHSCREEN_CONFIG_LEFT_BOTTOM)
		{
			tv_line (255,0,255,20,16);
			tv_line (245,0,255,0,16);	
			tv_line (0,251,10,251,128);
			tv_line (0,231,0,251,128);
		}
		else if (c==TOUCHSCREEN_CONFIG_RIGHT_BOTTOM)
		{
			tv_line (0,251,10,251,16);
			tv_line (0,231,0,251,16);
			tv_line (255,231,255,251,128);
			tv_line (245,251,255,251,128);	
		}
		else if (c==TOUCHSCREEN_CONFIG_OK)
		{
			tv_line (255,231,255,251,16);
			tv_line (245,251,255,251,16);		
		}
		else if (c==TOUCHSCREEN_CONFIG_TERMINAL)
			Terminal(TERMINAL_INTERMEDIATE); 
	}  
}


#define CAPTIONPOSITION_VERTICAL  20


void MessageBox(void)
{

    // Menue-Modus - MODE_MESSAGEBOX
    // Fenster mit  unterschiedlichen Darstellungsfunktionen
    // mit dem Code MODE_QUIT wird der Modus beendet
    // Weitere Steuerzeichen

    // #define MESSAGEBOX_BACK_NEXT		1		// Pfeile nach links und nach rechts
    // #define MESSAGEBOX_BACK_OK_NEXT		2		// Pfeile nach links und nach rechts, auerdem OK in der Mitte
    // #define MESSAGEBOX_OK_CANCEL        3		// OK und Abbrechen
    // #define MESSAGEBOX_YES_NO           4		// Ja und Nein
    // #define MESSAGEBOX_OK               5		// OK

    // #define MESSAGEBOX_SELECTBTN1		11		// linker Knopf gedrckt
    // #define MESSAGEBOX_SELECTBTN2		12		// mittlerer Knopf gedrckt
    // #define MESSAGEBOX_SELECTBTN3		13		// rechter Knopf gedrckt

    // #define MESSAGEBOX_LINE1			21		// Text in Zeile 1 ausgeben
    // #define MESSAGEBOX_LINE3			22		// Text in Zeile 2 ausgeben
    // #define MESSAGEBOX_LINE3			23		// Text in Zeile 3 ausgeben
    // #define MESSAGEBOX_LINE4			24		// Text in Zeile 4 ausgeben

	unsigned char c, xpos=16, ypos=10, MemoryPage;
	
// Touchscreen sendet Felder
	TouchScreen_SendFields;
	
	MemoryPage = PORTB;
	Mem_SetWindow2;
	
	LoescheBildschirm(WindowBkColor);
	
	for (;;)
	{
		c = uart_getchar();
		if (c==MODE_QUIT)		//Steuercode, um Funktion zu verlassen
			break;
		
		if (c >= 32)	// Text je nach Modus ausgeben
		{
			if (xpos < 240)
			{
				if (ypos == CAPTIONPOSITION_VERTICAL)
				{
					tv_writechar3x(xpos, ypos, TextColor, WindowBkColor, c); 
					xpos+=2*Pixelwidth;
				}	
				else
				{
					tv_writechar2x(xpos, ypos, TextColor, WindowBkColor, c); 		
					xpos+=Pixelwidth;
				}	
			}	
		}
		else if ((c>=MESSAGEBOX_BACK_NEXT) && (c<=MESSAGEBOX_OK))	// Fenster soll gezeichnet werden
		{
			if (c<MESSAGEBOX_OK)		// Fenster bentigt Knpfe links und rechts
			{
				// Knopf links
				DrawButton(3,190,93,247,0);
				
				// Knopf rechts
				DrawButton(163,190,253,247,0);
				
			}
			if ((c==MESSAGEBOX_BACK_OK_NEXT) || (c==MESSAGEBOX_OK))
			{
				DrawButton(98,190,158,247,0);
			}
			if ((c==MESSAGEBOX_BACK_NEXT) || (c==MESSAGEBOX_BACK_OK_NEXT))
			{
				WriteString(40, 212, TextColor, WindowBkColor, strBack);
				WriteString(200, 212, TextColor, WindowBkColor, strNext);
			}
			else if (c==MESSAGEBOX_OK_CANCEL)
			{
				WriteString(42, 212, TextColor, WindowBkColor, strOK);
				WriteString(182, 212, TextColor, WindowBkColor, strCancel);
			}
			else if (c==MESSAGEBOX_YES_NO)
			{
				WriteString(42, 212, TextColor, WindowBkColor, strYes);
				WriteString(196, 212, TextColor, WindowBkColor, strNo);
			}
			if ((c==MESSAGEBOX_BACK_OK_NEXT) || (c==MESSAGEBOX_OK))
			{
				WriteString(124, 212, TextColor, WindowBkColor, strOK);
			}
		}
		
		else if (c==MESSAGEBOX_SELECTBTN1)
			DrawButton(3,190,93,247,255);
		else if (c==MESSAGEBOX_SELECTBTN2)
			DrawButton(98,190,158,247,255);
		else if (c==MESSAGEBOX_SELECTBTN3)
			DrawButton(163,190,253,247,255);
		else if ((c>=MESSAGEBOX_LINE1) && (c<=MESSAGEBOX_LINE4))	// Zeile fr Textausgabe festlegen
		{
			if (c == MESSAGEBOX_LINE1)
				ypos = CAPTIONPOSITION_VERTICAL;
			else
				ypos = (c-MESSAGEBOX_LINE1)*32 + 50;
			xpos = 16;
		}
	}
	PORTB = MemoryPage;
}

void Draw_LED(unsigned char xpos, unsigned char ypos, unsigned char color)
{
	unsigned char x, y;
	for (y=ypos; y<=ypos+50; y++)
		for (x= xpos; x<= xpos + 26; x++)
			pixel[y][x]=color;	
}


#define LED8VERTICALPOSITION 	100

void LED_8fach(void)
{
	// Menue-Modus - MODE_8COLOR_LED
	// Acht Rechtecke in whlbaren Farben (blau, rot, grn und gelb) in horizontaler Anordnung
	// symbolisieren die acht roten LED's des Original-Microcontroller-Labs
	// mit dem Code MODE_QUIT wird der Modus beendet
	// Weitere Steuerzeichen

	// #define LED_8COLOR_BLUE				1		// Alle LEDs sind blau
	// #define LED_8COLOR_GREEN				2		// Alle LEDs sind gruen
	// #define LED_8COLOR_RED				3		// Alle LEDs sind rot
	// #define LED_8COLOR_YELLOW			4		// Alle LEDs sind gelb
	// #define LED_8COLOR_MIXED				5		// Je 2 LEDs sind blau, gruen, gelb, rot
	// #define LED_8COLOR_RAINBOW			6		// Regenbogen-Farbanordnung
	// #define LED_8COLOR_WHITE				7		// Alle LEDs sind wei
	// #define LED_8COLOR_CLEARSCREEN		8		// Lscht den Bildschirm
	// #define LED_8COLOR_LEDSWITCH			255		// einzelne LEDs einschalten - zweiter Wert erforderlich,
													// der die LEDs bitcodiert schaltet: LSB = links, MSB = rechts


	unsigned char i, c=0, xpos=16;
	unsigned char graf[8];
	unsigned char bkgraf[8];
// Touchscreen sendet Felder
	TouchScreen_SendFields;
	
	LoescheBildschirm(0);
	
	for (i=0;i<8;i++)
		Draw_LED(i*32 + 2, LED8VERTICALPOSITION, WindowBkColor);

	for (;;)
	{
		if ((c>=LED_8COLOR_BLUE) && (c<=LED_8COLOR_WHITE))
		{
			if	(c==LED_8COLOR_BLUE)	// Farben auf blau setzen
			{
				graf[0] = 1;
				bkgraf[0] = WindowBkColor;
			}		
			else if	(c==LED_8COLOR_GREEN)	// Farben auf grn setzen
			{
				graf[0] = 16;
				bkgraf[0] = 4;
			}	
			else if	(c==LED_8COLOR_RED)	// Farben auf rot setzen
			{
				graf[0] = 128;
				bkgraf[0] = 32;
			}	
			else if	(c==LED_8COLOR_YELLOW)	// Farben auf gelb setzen
			{
				graf[0] = 253;
				bkgraf[0] = 36;
			}	
			else // Farben auf wei setzen
			{
				graf[0] = 255;
				bkgraf[0] = 0;	
			}
			for (i=1;i<8;i++)
			{
				graf[i] = graf[0];
				bkgraf[i] = bkgraf[0];
			}	
		}
		else if	(c==LED_8COLOR_MIXED)	// Farben auf 2xblau,2xgrn,2xgelb,2xrot setzen
		{
			graf[0] = 1;	//blau
			bkgraf[0] = WindowBkColor;
			graf[1] = 1;	//blau
			bkgraf[1] = WindowBkColor;
			graf[2] = 16;	//grn
			bkgraf[2] = 4;
			graf[3] = 16;	// grn
			bkgraf[3] = 4;
			graf[4] = 253;	// gelb
			bkgraf[4] = 36;
			graf[5] = 253;	// gelb
			bkgraf[5] = 36;
			graf[6] = 128;	// rot
			bkgraf[6] = 32;
			graf[7] = 128;	// rot
			bkgraf[7] = 32;
		}	
		else if	(c==LED_8COLOR_RAINBOW)	// Farben auf Regenbogen setzen
		{
			graf[0] = 128;	//rot
			bkgraf[0] = 0;
			graf[1] = 224;	//orange
			bkgraf[1] = 0;
			graf[2] = 253;	//gelb
			bkgraf[2] = 0;
			graf[3] = 16;	// grn
			bkgraf[3] = 0;
			graf[4] = 31;	// hellblau
			bkgraf[4] = 0;
			graf[5] = 7;	// dunkelblau
			bkgraf[5] = 0;
			graf[6] = 102;	// lila
			bkgraf[6] = 0;
			graf[7] = 251;	// pink
			bkgraf[7] = 0;
		}
		else if (c==LED_8COLOR_CLEARSCREEN)
		{
			xpos = 16;
			LoescheBildschirm(0);
			c=254;
		}
		else if ((c >= 32) && (c < 128))
		{
			// Als Titeltext ausgeben, zweifache Schriftgre im oberen Bereich
			if (xpos < 240)
			{
				tv_writechar2x(xpos,24,TextColor,0,c);
				xpos+=Pixelwidth;
			}	
		}
		else if ((c>=LED_1_OFF) && (c<=LED_8_OFF))
		{
			c-= LED_1_OFF;
			Draw_LED(c*32 + 2, LED8VERTICALPOSITION, bkgraf[c]);
		}	
		else if ((c>=LED_1_ON) && (c<=LED_8_ON))
		{
			c-= LED_1_ON;
			Draw_LED(c*32 + 2, LED8VERTICALPOSITION, graf[c]);
		}	
		else if (c==LED_NOBKCOLOR)
		{
			for (i=0;i<8;i++)
				bkgraf[i] = WindowBkColor;
		}	
		c = uart_getchar();
		if (c==MODE_QUIT)
			break;
	}
	
}

void LED_Ampel(void)
{
    // Ampel-Mode - MODE_3COLOR_LED
    // Drei Rechtecke in den Farben rot, grn und gelb in Ampelanordnung
    // Symbolisieren die drei LED's des Original-Microcontroller-Labs
    // mit dem Code MODE_QUIT wird der Modus beendet
    // Weitere Steuerzeichen
    // #define LED_3COLOR_GREEN_ON    2 // gruen hell
    // #define LED_3COLOR_GREEN_OFF   3 // gruen dunkel
    // #define LED_3COLOR_YELLOW_ON   4 // gelb hell
    // #define LED_3COLOR_YELLOW_OFF  5 // gelb dunkel
    // #define LED_3COLOR_RED_ON      6 // rot hell
    // #define LED_3COLOR_RED_OFF     7 // rot dunkel

    
	unsigned char c=3, i, ypos, graf[3]={32, 36, 4};
    
// Touchscreen sendet Felder
	TouchScreen_SendFields;
	
	LoescheBildschirm(0);
	for (;;) 
	{
		if ((c>=LED_3COLOR_GREEN_ON) && (c<=LED_3COLOR_RED_OFF))
        {
            if (c == LED_3COLOR_GREEN_ON)	// gruen ein
                graf[2] = 16; 	// helles Gruen
            else if (c == LED_3COLOR_GREEN_OFF)	// grn aus
                graf[2] = 4; // dunkles Gruen
            
            else if (c == LED_3COLOR_YELLOW_ON)	// gelb ein
                graf[1] = 253; 	// helles Gelb
            else if (c == LED_3COLOR_YELLOW_OFF)	// gelb aus
                graf[1] = 36; // dunkles Gelb

            else if (c == LED_3COLOR_RED_ON)	// rot ein
                graf[0] = 128; 	// helles Rot
            else if (c == LED_3COLOR_RED_OFF)	// rot aus
                graf[0] = 32; // dunkles Rot
  
            ypos = 6;
            for (i=0; i<3; i++)
            {
                Draw_LED(108, ypos, graf[i]);	
                ypos+=84;
            }
        }
		c = uart_getchar();
		
		if (c==MODE_QUIT)
			break;	
			
	}
}



void FarbPalette(void)
{
    // Stellt die gesamte verfgbare Farbpalette dar - MODE_COLORPALETTE
    // mit dem Code MODE_QUIT wird der Modus beendet
	// Toucchscreen sendet Koordinaten an die Hauptanwendung
	// Fr eine echte Farbauswahl muss der Bildschirm exakt kalibriert sein.
	// Es wird auf eine gesonderte Speicherseite geschaltet, um den Inhalt
	// der Hauptseite nicht zu zerstren
	unsigned char c, i, j, x, y, MemoryPage;
	
	
	
	MemoryPage = PORTB;
	Mem_SetWindow3;
	
	LoescheBildschirm(0);
    TouchScreen_SendCoordinates;
	
	for (i=0;i<16;i++)
	{
		for (j=0;j<16;j++)
		{
			for (y=i*16; y<=i*16+8; y++)
				for (x=j*16; x<=j*16+12; x++)
					pixel[y][x]=255 - (j + i*16);
		}
	}
	for (;;)
	{
		c = uart_getchar();
		if ((c==MODE_QUIT)	|| (c==MODE_END_INTERMEDIATE))// Abbruchkriterium
			break;
	}	
	PORTB = MemoryPage;	 
}


void FreieGrafik(void)
{
    // Freier Zeichenmodus - MODE_DRAW
    // mit dem Code MODE_QUIT wird der Modus beendet
    // Weitere Steuerzeichen
    // #define DRAW_CLEARSCREEN        2 		// Bildschirm loeschen
    // #define DRAW_COLOR_RGB          4 		// Zeichenfarbe als RGB (weitere 3 Werte fr rot, gruen, blau erforderlich)
    // #define DRAW_COLOR              5		// Zeichenfarbe als Wert (ein weiterer einzelner Wert zwischen 0 und 255)
    // #define DRAW_BKCOLOR            7		// Hintergrundfarbe als Wert (ein weiterer einzelner Wert zwischen 0 und 255)
    // #define DRAW_LINE               14		// Linie, erfordert vier weitere Werte fr x1/y1 und x2/y2
    // #define DRAW_SETPIXEL           15 		// Pixel, erfordert zwei weitere Werte fr x/y
    // #define DRAW_DRAWRECT           19		// Rechteck, erfordert vier weitere Werte fr x1/y1 und x2/y2
    // #define DRAW_FILLRECT           20		// Geflltes Rechteck, erfordert vier weitere Werte fr x1/y1 und x2/y2
    // #define DRAW_BUTTONFRAME    	   21		// zeichnet einen transparenten Knopf (links oben: hell, rechts unten: dunkel
    // #define DRAW_SELBUTTONFFRAME    22		// zeichnet einen selektierten Knopf (links oben: dunkel, rechts unten: hell
    // #define DRAW_SWITCHTERMINAL     23       // schaltet temporr in den Terminalmodus, damit Texte ausgegeben werden knnen
    
	unsigned char c, graf=255, bkgraf=0, MemoryPage;
 
	TouchScreen_SendCoordinates;
	
	MemoryPage = PORTB;
	Mem_SetWindow1;
	
	LoescheBildschirm(0);
	for (;;)
	{
		c = uart_getchar();
            
		if (c==DRAW_CLEARSCREEN)		// Loesche Bildschirm
			LoescheBildschirm(bkgraf);
        
/*		else if (c==DRAW_COLOR_RGB)			//Set Grafic Color (RGB)
		{	unsigned char r,g,b;
			r=uart_getchar();	
			g=uart_getchar();
			b=uart_getchar();
			graf=RGB(r,g,b);
		} */
		else if (c==DRAW_COLOR)			//Set Grafic Color (Color Value)
			graf=uart_getchar();
        else if (c==DRAW_BKCOLOR)			//Set Background Color (Color Value)
            bkgraf=uart_getchar();
		else if (c==DRAW_SELECTCOLOR) 
		{
			FarbPalette();
			TouchScreen_SendCoordinates;
			Mem_SetWindow1;
		}	
		else if (c==DRAW_LINE)			//Draw Line
		{	unsigned char x1,y1,x2,y2;
		
			x1=uart_getchar();
			y1=uart_getchar();
			x2=uart_getchar();
			y2=uart_getchar();
			tv_line (x1,y1,x2,y2,graf);
		}
		else if (c==DRAW_SETPIXEL)			//Set Pixel
		{	unsigned char x,y;
			x=uart_getchar();
			y=uart_getchar();
			if (y>=YSIZE)
				y=YSIZE-1;
			pixel[y][x]=graf;
		}
		else if (c==DRAW_DRAWRECT)			//Draw rect
		{	unsigned char x1,y1,x2,y2;
			x1=uart_getchar();
			y1=uart_getchar();
			x2=uart_getchar();
			y2=uart_getchar();
			tv_line (x1,y1,x2,y1,graf);
			tv_line (x2,y1,x2,y2,graf);
			tv_line (x2,y2,x1,y2,graf);
			tv_line (x1,y2,x1,y1,graf);
		}
		else if (c==DRAW_FILLRECT)			//Draw filled rect
		{	unsigned char x1,y1,x2,y2,y,x;
			x1=uart_getchar();
			y1=uart_getchar();
			x2=uart_getchar();
			y2=uart_getchar();
			if (y1>=YSIZE)
				y1=YSIZE-1;
			if (y2>=YSIZE)
				y2=YSIZE-1;
			for (y=y1; y<=y2; y++)
				for (x=x1; x<=x2; x++)
					pixel[y][x]=graf;
			
		}
		else if ((c==DRAW_BUTTONFFRAME) || (c==DRAW_SELBUTTONFFRAME))
		{	unsigned char x1,y1,x2,y2;
			x1=uart_getchar();
			y1=uart_getchar();
			x2=uart_getchar();
			y2=uart_getchar();
			if (c==DRAW_BUTTONFFRAME)
				DrawButton(x1, y1, x2, y2, 0);
			else	
				DrawButton(x1, y1, x2, y2, 255);
		}
		
        else if (c==DRAW_SWITCHTERMINAL)
        {
            Terminal(TERMINAL_INTERMEDIATE);
            TouchScreen_SendCoordinates;
			Mem_SetWindow1;
        }
		else if (c==MODE_QUIT)
			break;
	}	
	
	PORTB = MemoryPage;
	
}




/*

void Bitmap(void)
{

// Stellt ein Bitmap ab der Position 0,0 dar
	unsigned char x1,y1,xs,ys,x,y;
	unsigned char c;
	
	LoescheBildschirm(0);
	
	x1=uart_getchar();
	y1=uart_getchar();
	xs=uart_getchar();
	ys=uart_getchar();
	for (y=0; y<ys; y++)
	{
		for (x=0; x<xs; x++)
		{	
			c=uart_getchar();
			if ((y1+y)<YSIZE)
				pixel[y+y1][x+x1]=c;
		}
	}	
	for (;;)
	{
		c = uart_getchar();
		if (c==31)	// Abbruchkriterium
			break;
	}	

}

*/


void Schieberegler(void)
{
	// Schieberegler - MODE_SLIDER
	// mit dem Code MODE_QUIT wird der Modus beendet	
	// Werte ber 31 werden als Text interpretiert und zweizeilig oben ausgegeben
	// Weitere Steuerzeichen
	// #define SLIDER_REDRAW				1	// Neuzeichnen im unselektierten Zustand
	// #define SLIDER_BTN_BACK_PRESSED		2	// Back-Button gedrckt
	// #define SLIDER_BTN_NEXT_PRESSED		3	// Next-Button gedrckt
	// #define SLIDER_BTN_OK_PRESSED		4	// OK-Button gedrckt
	// #define SLIDER_VALUE					5	// Wert fr Slider - zweiter Wert wird abgefragt, zulssig sind Werte von 0 bis 200
	// #define SLIDER_CLEARSCREEN			6	// lscht den Bildschirm inklusive des Textes
	
	unsigned char i, x, y, c=1, xpos=16, ypos=24, value=50, MemoryPage;
	
	MemoryPage = PORTB;
	Mem_SetWindow3;
	
	LoescheBildschirm(WindowBkColor);
    TouchScreen_SendCoordinates;
	
	WriteString(6, 125, TextColor, WindowBkColor, strBack);
	WriteString(236, 125, TextColor, WindowBkColor, strNext);
	WriteString(124, 212, TextColor, WindowBkColor, strOK);
	
	for (;;)
	{	
		if (c==SLIDER_REDRAW)
		{
			DrawButton(25,125,228,133,0);	// Slider
			DrawButton(98,190,158,247,0);	// OK-Knopf
			DrawButton(0,111,24,152,0);	// Knopf links
			DrawButton(230,111,254,152,0);	// Knopf rechts
			DrawButton(28 + value,115,33 + value,143,0);	// Slider-Knopf
			for (i=0; i<4; i++)
				tv_line(29 + value + i,117,29 + value + i, 141, WindowBkColor);
			c=0;		// Zurcksetzen, weil sonst keine Abfragen mehr erfolgen
		}	
		else if (c==SLIDER_BTN_OK_PRESSED)
			DrawButton(98,190,158,247,255);	// OK gedrckt
		else if (c==SLIDER_BTN_BACK_PRESSED)	
			DrawButton(0,111,24,152,255);		// links gedrckt
		else if (c==SLIDER_BTN_NEXT_PRESSED)
			DrawButton(230,111,254,152,255);	// rechts gedrckt
		else if (c==SLIDER_VALUE)
		{
			y = 114;
			while (y++ < 144)
			{
				x = value + 27;
				while (x++ < value + 34)
					pixel[y][x] = WindowBkColor;
			}	
			value = uart_getchar();	
			if (value > 199)
				value = 199;
			c=SLIDER_REDRAW;	// Neuzeichnen anstoen
		}	
		else if (c==SLIDER_CLEARSCREEN)
		{
			LoescheBildschirm(WindowBkColor);
			c=SLIDER_REDRAW;	// Neuzeichnen anstoen
		}	
		else if (c>32)
		{
			if ((xpos < 240) && (ypos < 50))
			{
				tv_writechar2x(xpos, ypos,TextColor,WindowBkColor,c);
				xpos+=Pixelwidth;
				if (xpos >= 240)
				{
					ypos+=24;
					xpos=16;
				}
			}	
		}		
				
		if (c != SLIDER_REDRAW) 	// Um Neuzeichnen zu ermglichen			
			c = uart_getchar();
		if (c==MODE_QUIT)	// Abbruchkriterium
			break;		
			
	}	
	PORTB = MemoryPage;	
}


int main(void)
{
//	unsigned char xpos=255;
	PORTD = 0xFE;  // D0 ist Input fr serielle Befehle
// Touchscreen-Modus am angeschlossenen Tastaturcontroller setzen
// D3=0, D5=0 - Touchscreen sendet keine Befehle
// D3=1, D5=0 - Touchscreen sendet 8x6 Felder
// D3=0, D5=1 - Touchscreen sendet Koordinaten
// D3=1, D5=1 - Touchscreen wird kalibriert
	// Touchscreen sendet keine Befehle
	TouchScreen_TurnOff;	
	
// Speicherausgabeseite setzen
// Funktion ist bisher nicht implementiert	
// 4 Seiten sind moeglich
// Seite 0 - Textausgabe / Grafik
// Seite 1 - Messwerte (knnen damit dauerhaft gespeichert werden
// Seite 2 - Menueauswahl / Messagebox (berschreibt damit nicht andere Ausgaben)
// Seite 3 - sonstige Grafiken
	DDRB |= (1<<PB1) | (1<<PB0);
	// Seite 0 wird gesetzt
	Mem_SetWindow0;


	tv_init();	
//TV Generator mit Speicherinterface, Timer1 und Ports initialisieren
	uart_init(UART_BAUD_SELECT(UART_BAUD_RATE, XTAL_CPU));

// Grafik-Modi	
    // #define MODE_TERMINAL                0	// Terminalmodus
    // #define MODE_BALKENGRAFIK            1	// Balkengrafik
    // #define MODE_MESSWERTE               2   // Messwerte
    // #define MODE_MENU                    3   // Menue
    // #define MODE_KEYBOARD_ALPHA          4   // Bildschirmtastatur fr Texteingabe
    // #define MODE_KEYBOARD_NUMERIC        5   // Bildschirmtastatur fr Zahleneingabe
    // #define MODE_MESSAGEBOX				6 	// Auswahlfenster Ja-Nein / Ok-Abbrechen
    // #define MODE_8COLOR_LED				7 	// 8fach-LED-Bildschirm
    // #define MODE_3COLOR_LED              8	// 3fach-LED in Ampelanordnung und -farbe
    // #define MODE_DRAW                    9  	// Freie Grafiken zeichnen
    // #define MODE_COLORPALETTE			10 	// Farbdemo - Rechtecke mit Farbe aufsteigend von 0..255
    // #define MODE_SLIDER					11 	// Schieberegler
	// 12 - Bitmap anzeigen - nicht freigeschaltet 
	// #define MODE_LAST = (letzter gltiger Modus)

	// Grafik-Modus "Terminal" wird aktiviert
	unsigned int graphics_mode = MODE_TERMINAL;
	
	for (;;)
	{	
		// Die einzelnen Grafikmodi bleiben solange erhalten, bis das Zeichen 31 gesendet wird
		if (graphics_mode == MODE_TERMINAL)
		{
			Terminal(0);
		}
		else if (graphics_mode == MODE_BALKENGRAFIK)
		{	
			Balkengrafik();
			
		}
		else if (graphics_mode == MODE_MESSWERTE)
		{	
			//xpos = Messwerte(xpos);
			Messwerte();
		}
		else if (graphics_mode == MODE_MENU)
		{	
			Menu();
			
		}
		else if (graphics_mode == MODE_KEYBOARD_ALPHA)
		{	
			Texteingabe(KEYBOARD_ALPHA);
			
		}
		else if (graphics_mode == MODE_KEYBOARD_NUMERIC)
		{	
            Texteingabe(KEYBOARD_NUMERIC);
		}
		else if (graphics_mode == MODE_MESSAGEBOX)
		{	
			MessageBox();
		}
		else if (graphics_mode == MODE_8COLOR_LED)
		{	
			LED_8fach();
			
		}
		else if (graphics_mode == MODE_3COLOR_LED)
		{	
			LED_Ampel();		
		}
		else if (graphics_mode == MODE_DRAW)
		{	
			FreieGrafik();
//			xpos = 255; 	// nutz das gleiche MemWindow wie Messwerte, die sind danach unbrauchbar
			
		}
/*		
		else if (graphics_mode == 12)
		{	
			Bitmap();			
		}
*/		
		else if (graphics_mode == MODE_COLORPALETTE)
		{	
			FarbPalette();
			
		}
		else if (graphics_mode == MODE_SLIDER)
		{	
			Schieberegler();
		}
		else if (graphics_mode == MODE_TOUCHSCREEN_CONFIG)
		{	
			Kalibrierung();
		}
		else if (graphics_mode == MODE_KEYBOARD_DRAW)
		{	
			Texteingabe(KEYBOARD_DRAW);
		}	
		// Dieser Teil wird nur erreicht, wenn einer der Grafikmodi verlassen wurde
		// das nchste Zeichen ist dann der neue Modus
		graphics_mode = uart_getchar();
		if (graphics_mode > MODE_LAST)
			graphics_mode = 0;	// Falls ungltiger Modus, dann Terminal aktivieren
	}	
}


