// VGA Microcontroller
// for Philips VGA Module 


#define F_CPU 				16000000
#define XTAL_CPU			16000000
#define USART_BAUD_RATE		   19200
#define UART_BAUD_RATE		   19200 
#define MY_UBRR   F_CPU/16/USART_BAUD_RATE-1

#include <stdlib.h> 
#include <util/delay.h>
#include <avr/io.h>
#include <uart.c>
#include <avr/interrupt.h>
#include "SendKeys.h"
#include "grafik.h"
#include "calculator.c"
// #include "mclib.c"


const char str_ProgramName[]="Microcontroller Lab";
const char str_VersionProgram[]="Version 2.0.017";
const char str_VersionGraphics[]="Grafikkarte Version 1.1.032";
const char str_VersionTouchScreen[]="Touchscreen Version 1.0.018";
const char str_Copyright[]="(c) 2013 Frank Brennecke";







void Show_Boot_Info(void)
{
//  Boot-Texte ausgeben   
    Send_Command(MODE_QUIT);  // Aktuellen Modus verlassen
    Send_Command(MODE_TERMINAL);   // Terminal-Modus    
    Send_Command(TERMINAL_FONT_BIG);
	Send_Command(TERMINAL_TEXTCOLOR); 
	Send_Command(128);  // rot 
    Send_String(str_ProgramName); 
	Send_Command(TERMINAL_TEXTCOLOR);
	Send_Command(255);  // wei 
    Send_Command(TERMINAL_CARRIAGERETURN); Send_Command(TERMINAL_LINEFEEED);  
	Send_Command(TERMINAL_CARRIAGERETURN); Send_Command(TERMINAL_LINEFEEED); 
	Send_Command(TERMINAL_CARRIAGERETURN); Send_Command(TERMINAL_LINEFEEED); 
    Send_Command(TERMINAL_FONT_MEDIUM);
    Send_String(str_VersionProgram);     
    Send_Command(TERMINAL_CARRIAGERETURN); Send_Command(TERMINAL_LINEFEEED);      
    Send_Command(TERMINAL_CARRIAGERETURN); Send_Command(TERMINAL_LINEFEEED);
    Send_String(str_Copyright);   
    Send_Command(TERMINAL_CARRIAGERETURN); Send_Command(TERMINAL_LINEFEEED);     
    Send_Command(TERMINAL_CARRIAGERETURN); Send_Command(TERMINAL_LINEFEEED);
	_delay_ms(200);
	Send_String(str_VersionGraphics);   
    Send_Command(TERMINAL_CARRIAGERETURN); Send_Command(TERMINAL_LINEFEEED);     
    Send_Command(TERMINAL_CARRIAGERETURN); Send_Command(TERMINAL_LINEFEEED);
	_delay_ms(200);
	Send_String(str_VersionTouchScreen);   
    Send_Command(TERMINAL_CARRIAGERETURN); Send_Command(TERMINAL_LINEFEEED);     
    Send_Command(TERMINAL_CARRIAGERETURN); Send_Command(TERMINAL_LINEFEEED);
    _delay_ms(1000);
}




const char str_WelcomeCaption[]="Willkommen!";
const char str_WelcomeLine1[]="Dieses Programm demonstriert einzelne";
const char str_WelcomeLine2[]="Funktionen und deren Mglichkeiten.";
const char str_WelcomeLine3[]="Weiter? Tippen auf <OK>";



void Show_Welcome_Screen(void)
{
	MessageBox(MESSAGEBOX_OK, str_WelcomeCaption, str_WelcomeLine1, str_WelcomeLine2, str_WelcomeLine3);
}









void Kalibrierung(void)
{
	unsigned char c=0;
//	char String_Buffer[STRING_BUFFER_SIZE];


	Send_Command(MODE_QUIT);  // Aktuellen Modus verlassen
	Send_Command(MODE_TOUCHSCREEN_CONFIG);
	
	Send_Command(TOUCHSCREEN_CONFIG_TERMINAL);
	Send_Terminal_String(12,24,0,"Linke obere Ecke antippen... "); 
	Send_Command(MODE_END_INTERMEDIATE);
	
	Send_Command(TOUCHSCREEN_CONFIG_LEFT_TOP);
	
	for(;;)
	{
		c = Check_For_Input;
		if (c > 0)
		{
			if (c==SENDKEY_CONFIG_2)
			{
				Send_Command(TOUCHSCREEN_CONFIG_TERMINAL);
				Send_Terminal_String(198, 24, 0,"OK");
				_delay_ms(200);
				Send_Terminal_String(12,40, 0, "Rechte obere Ecke antippen... "); 
				Send_Command(MODE_END_INTERMEDIATE);
				Send_Command(TOUCHSCREEN_CONFIG_RIGHT_TOP);
			}	
			else if (c==SENDKEY_CONFIG_3)
			{
				Send_Command(TOUCHSCREEN_CONFIG_TERMINAL);
				Send_Terminal_String(198, 40, 0, "OK");
				_delay_ms(200);
				Send_Terminal_String(12,56, 0, "Linke untere Ecke antippen... ");
				Send_Command(MODE_END_INTERMEDIATE);
				Send_Command(TOUCHSCREEN_CONFIG_LEFT_BOTTOM);	
			}	
			else if (c==SENDKEY_CONFIG_4)
			{
				Send_Command(TOUCHSCREEN_CONFIG_TERMINAL);
				Send_Terminal_String(198, 56, 0, "OK");
				_delay_ms(200);
				Send_Terminal_String(12,72, 0, "Rechte untere Ecke antippen... ");
				Send_Command(MODE_END_INTERMEDIATE);
				Send_Command(TOUCHSCREEN_CONFIG_RIGHT_BOTTOM);	
			}	
			else if	((c==SENDKEY_RESTART_CONFIG_X) || (c==SENDKEY_RESTART_CONFIG_Y) || (c==SENDKEY_RESTART_CONFIG_POSITION))
			{
				Send_Command(TOUCHSCREEN_CONFIG_TERMINAL);
				Send_Terminal_String(12,24,1, "Eingabe ungenau...");
				Send_Terminal_String(12,40,1, " ");
				Send_Terminal_String(12,56,1, " ");
				Send_Terminal_String(12,72,1, " ");
				_delay_ms(1000);
				Send_Terminal_String(16,24,0,"Linke obere Ecke antippen... ");
				Send_Command(MODE_END_INTERMEDIATE);
				Send_Command(TOUCHSCREEN_CONFIG_LEFT_TOP);
			}
			else if (c==SENDKEY_CONFIG_COMPLETE)
			{
				Send_Command(TOUCHSCREEN_CONFIG_OK);
				Send_Command(TOUCHSCREEN_CONFIG_TERMINAL);
				Send_Terminal_String(12,24,1,"Kalibrierung abgeschlossen");
				Send_Terminal_String(12,40,1," ");
				Send_Terminal_String(12,56,1,"Weiter mit OK-Taste");
				Send_Terminal_String(12,72,1," ");
				Send_Command(MODE_END_INTERMEDIATE);
			}
			else if (c==SENDKEY_OK)
				break;
			
		}	
	}
}




void Show_Serial_Debug_Screen(void)
{
	unsigned char c=0;
//	char String_Buffer[STRING_BUFFER_SIZE];

	Send_Command(MODE_QUIT);  // Aktuellen Modus verlassen
	Send_Command(MODE_DRAW);   // Freies Zeichnen  
	Send_Command(DRAW_SWITCHTERMINAL);  // In Terminal-Modus umschalten
	Send_String("Debug");
    Send_Command(13); Send_Command(10); 
	for(;;)
	{
		c = Check_For_Input;
		_delay_ms(10);
		
		
		switch (c)
		{
			case SENDKEY_REQUIRES_CALIBRATION: Send_String(" SENDKEY_REQUIRES_CALIBRATION "); break;
			case SENDKEY_TOUCH_END: Send_String(" SENDKEY_TOUCH_END "); break;
			case SENDKEY_TOUCH_START: Send_String(" SENDKEY_TOUCH_START "); break;
			case SENDKEY_OK: Send_String(" SENDKEY_OK "); break;
			case SENDKEY_LEFT: Send_String(" SENDKEY_LEFT "); break;
			case SENDKEY_RIGHT: Send_String(" SENDKEY_RIGHT "); break;
			case SENDKEY_FIELDS: Send_String(" SENDKEY_FIELDS "); break;
			case SENDKEY_COORDINATES: Send_String(" SENDKEY_COORDINATES "); break;
			case SENDKEY_SILENT: Send_String(" SENDKEY_SILENT "); break;
			case SENDKEY_CALIBRATION: Send_String(" SENDKEY_CALIBRATION "); break;
			case SENDKEY_STARTUP: Send_String(" SENDKEY_STARTUP "); break;
			case SENDKEY_ERROR: Send_String(" SENDKEY_ERROR "); break;
			case SENDKEY_SLEEP: Send_String(" SENDKEY_SLEEP "); break;
			case SENDKEY_CONFIG_2: Send_String(" SENDKEY_CONFIG_2 "); break;
			case SENDKEY_CONFIG_3: Send_String(" SENDKEY_CONFIG_3 "); break;
			case SENDKEY_CONFIG_4: Send_String(" SENDKEY_CONFIG_4 "); break;
			case SENDKEY_RESTART_CONFIG_POSITION: Send_String(" SENDKEY_RESTART_CONFIG_POSITION "); break;
			case SENDKEY_CONFIG_COMPLETE: Send_String(" SENDKEY_CONFIG_COMPLETE "); break;
			case SENDKEY_RESTART_CONFIG_X: Send_String(" SENDKEY_RESTART_CONFIG_X "); break;
			case SENDKEY_RESTART_CONFIG_Y: Send_String(" SENDKEY_RESTART_CONFIG_Y "); break;
		}
		/*
		if (c>0)
			Send_String(_utoa(String_Buffer,STRING_BUFFER_SIZE,c));
		if ((c >= SENDKEY_TOUCH_START) && (c < SENDKEY_TOUCH_END))
			Send_String(_utoa(String_Buffer,STRING_BUFFER_SIZE,c));
		if ((c >= SENDKEY_TOUCH_END) && (c < SENDKEY_TOUCH_END+48))
			Send_String(_utoa(String_Buffer,STRING_BUFFER_SIZE,c));
		
		if (c > 0)
			Send_Command(c);
		*/	
		
	}
	Send_Command(MODE_END_INTERMEDIATE);  // Terminal-Modus verlassen
}


#define	TR_GREEN			1
#define	TR_BLINKYELLOWON	2
#define	TR_RED				3
#define	TR_BLINKYELLOWOFF	4

	
void SchalteAmpel(char light)	
{
	_delay_ms(200);
	if (light==TR_GREEN)
	{
		Send_Command(LED_3COLOR_GREEN_OFF);
		Send_Command(LED_3COLOR_RED_ON);
		Send_Command(LED_3COLOR_YELLOW_ON);
		_delay_ms(1000);
		Send_Command(LED_3COLOR_RED_OFF);
		Send_Command(LED_3COLOR_YELLOW_OFF);
		Send_Command(LED_3COLOR_GREEN_ON);
	}
	else if (light==TR_RED)
	{
		Send_Command(LED_3COLOR_GREEN_OFF);
		Send_Command(LED_3COLOR_YELLOW_ON);
		Send_Command(LED_3COLOR_RED_OFF);
		_delay_ms(1000);
		Send_Command(LED_3COLOR_RED_ON);
		Send_Command(LED_3COLOR_YELLOW_OFF);
		Send_Command(LED_3COLOR_GREEN_OFF);
	}
	else if (light==TR_BLINKYELLOWON)
	{
		Send_Command(LED_3COLOR_GREEN_OFF);
		Send_Command(LED_3COLOR_YELLOW_ON);
		Send_Command(LED_3COLOR_RED_OFF);
	}
	else if (light==TR_BLINKYELLOWOFF)
	{
		Send_Command(LED_3COLOR_GREEN_OFF);
		Send_Command(LED_3COLOR_YELLOW_OFF);
		Send_Command(LED_3COLOR_RED_OFF);
	}
}



void Ampelsteuerung(void)
{
	unsigned char c=0;
	unsigned char manual_lights;
	unsigned char current_light = TR_GREEN;
		
	manual_lights =  MessageBox(MESSAGEBOX_YES_NO, "Ampelsteuerung", "Soll die Ampel manuell","per Touch gesteuert werden?", "");
	
	Send_Command(MODE_QUIT);  // Aktuellen Modus verlassen
    Send_Command(MODE_3COLOR_LED);   // Ampel-LED  
	Send_Command(LED_3COLOR_GREEN_ON);	// grn an
	
	if (manual_lights == MR_YES)
	{
		for(;;)
		{
			c = Check_For_Input;
			if (c==SENDKEY_OK)
				break;	// Programm beenden
			else if ((c==SENDKEY_LEFT) && (current_light != TR_RED)) 
			{
				current_light = TR_RED;
				SchalteAmpel(current_light);
			}	
			else if ((c==SENDKEY_RIGHT) && (current_light != TR_GREEN)) 
			{
				current_light = TR_GREEN;
				SchalteAmpel(current_light);
			}	
			else if ((c>=SENDKEY_TOUCH_START) && (c<=SENDKEY_TOUCH_START+15) && (current_light != TR_RED))
			{
				current_light = TR_RED;
				SchalteAmpel(current_light);
			}
			else if ((c>=SENDKEY_TOUCH_START+16) && (c<=SENDKEY_TOUCH_START+31))
			{
				if ((current_light != TR_BLINKYELLOWON) && (current_light != TR_BLINKYELLOWOFF))
				{
					current_light = TR_BLINKYELLOWON;
					SchalteAmpel(current_light);
				}	
			}
			else if ((c>=SENDKEY_TOUCH_START+32) && (c<=SENDKEY_TOUCH_START+47) && (current_light != TR_GREEN))
			{
				current_light = TR_GREEN;
				SchalteAmpel(current_light);
			}
		
		
			if (current_light == TR_BLINKYELLOWON)
			{
				current_light = TR_BLINKYELLOWOFF;
				SchalteAmpel(current_light);
				_delay_ms(500);
			}
			if (current_light == TR_BLINKYELLOWOFF)
			{
				current_light = TR_BLINKYELLOWON;
				SchalteAmpel(current_light);
				_delay_ms(500);
			}
		}	
	}
	else
	{
// automatische Ablaufsteuerung
		for(;;)
		{
			c = Check_For_Input;
			if (c==SENDKEY_OK)
				break;	// Programm beenden	
			SchalteAmpel(TR_RED);
			_delay_ms(2000);
			SchalteAmpel(TR_GREEN);
			c = Check_For_Input;
			if (c==SENDKEY_OK)
				break;	// Programm beenden	
			_delay_ms(2000);	
		}	
	}
}











void Balkengrafik(void)
{
	unsigned char NumBalken;
	unsigned char c=BALKENGRAFIK_BAROFFSET, i=0, Balken[8];
	
	NumBalken = ShowKeyboard(1,8, NUMERIC_KEYBOARD, "Anzahl Balken (1-8) : ");
	
	if ((NumBalken > 1) && (NumBalken <= 8))
	{
		Send_Command(MODE_QUIT);
		Send_Command(MODE_BALKENGRAFIK);
		Send_Command(NumBalken);	// BALKENGRAFIK_NUMACHSEN_1			
		
		Send_Command(BALKENGRAFIK_COLOR_ORANGE);
		
		for (i=0; i < 8; i++)
			Balken[i] = BALKENGRAFIK_BAROFFSET; 		
			
			
		for(;;)
		{	
			for (i=0; i < NumBalken; i++)
			{
				if (Balken[i] >  251 - (i+1)*5)
					Balken[i] = BALKENGRAFIK_BAROFFSET;
				else
					Balken[i] += (i+1)*5; 
				if (Balken[i] > 251)
					Balken[i] =251;
				else if (Balken[i] <  BALKENGRAFIK_BAROFFSET)
					Balken[i] = BALKENGRAFIK_BAROFFSET;
			
				Send_Command(BALKENGRAFIK_DRAWBAR_1+i);
				Send_Command(Balken[i]);
				_delay_ms(5);
			}	
			c = Check_For_Input;
			if (c==SENDKEY_OK)	// OK-Taste bettigt
				break;
		}
	}	
	_delay_ms(200);	
}




void Farbauswahl(void)
{
	unsigned char c=0;
	Send_Command(MODE_QUIT);  // Aktuellen Modus verlassen
    Send_Command(MODE_COLORPALETTE);   // Farb-Demo    
	for(;;)
	{
		c = Wait_For_Input;
		if (c==SENDKEY_OK)	// OK-Taste bettigt
			break;
	}
}







void Lauflicht(void)
{
	unsigned char c=0, menu, menu2, i, inversion=0, color=LED_8COLOR_RED, direction=0;
	
	/*
	for (;;)
	{
		menu1 = Show_Menu_Screen("Lauflicht", "Farbe auswhlen ", "Umgekehrte Laufrichtung auswhlen ", "Invertierte Darstellung auswhlen", "Lauflicht starten", "Zurck zum Hauptmen");
		switch (menu)
		{
			case MENUITEM1:
	
	*/
	
	for (;;)
	{
		
		menu = Show_Menu_Screen("Lauflicht", "Rotes Lauflicht starten", "Blaues Lauflicht starten", "Grnes Lauflicht starten", "Mehrfarbiges Lauflicht starten", "Zurck zum Hauptmen");


		
		Send_Command(MODE_QUIT);  // Aktuellen Modus verlassen
		Send_Command(MODE_8COLOR_LED);   // Farb-Demo    
		Send_String("Lauflicht-Demo"); 
		
		
		switch (menu)
		{
			case MENUITEM1:
			{
				Send_Command(LED_8COLOR_RED);
				break;
			}
			case MENUITEM2:
			{
				Send_Command(LED_8COLOR_BLUE);
				break;
			}
			case MENUITEM3:
			{
				Send_Command(LED_8COLOR_GREEN);
				break;
			}
			case MENUITEM4:
			{
				Send_Command(LED_8COLOR_RAINBOW);
				break;
			}
		}
		if (menu==MENUITEM5)
			break;
		else if (menu==MENUITEM4)
		{
			i=0;
			for (;;)
			{
				Send_Command(LED_1_ON + i);
				if (++i > 7)
					i=0;
				Send_Command(LED_1_OFF + i);
				_delay_ms(100);
				c = Check_For_Input;
				if (c==SENDKEY_OK)	// OK-Taste bettigt
					break;
			}
		}
		else
		{
			i=0;
			for (;;)
			{
				Send_Command(LED_1_OFF + i);
				if (++i > 7)
					i=0;
				Send_Command(LED_1_ON + i);
				_delay_ms(100);
				c = Check_For_Input;
				if (c==SENDKEY_OK)	// OK-Taste bettigt
					break;
			}
		}
		
	}
}

#define DRAW 0
#define PALETTE 1
#define TEXT 2

#define COORDINATES_BUTTON 1,230,86,249

void Zeichnen(void)
{
	unsigned char c=0,x,y,xalt=0,yalt=0, firstcoordinate=1, mode=DRAW, button_selected = 0, SelectedColor=255;
	Send_Command(MODE_QUIT);  // Aktuellen Modus verlassen
	Send_Command(MODE_DRAW);   // Freies Zeichnen  
	Send_Command(DRAW_SWITCHTERMINAL);  // In Terminal-Modus umschalten
	Send_Command(TERMINAL_FONT_MEDIUM);
	Send_String("Mit dem Stift auf dem Bildschirm zeichnen"); 
	Send_Command(TERMINAL_CARRIAGERETURN); Send_Command(TERMINAL_LINEFEEED); 
	Send_Command(TERMINAL_CARRIAGERETURN); Send_Command(TERMINAL_LINEFEEED); 
	Send_String("Taste <links> - Bild lschen"); 
    Send_Command(TERMINAL_CARRIAGERETURN); Send_Command(TERMINAL_LINEFEEED);  
	Send_String("Taste <rechts> - Farbauswahl"); 
	Send_Command(TERMINAL_CARRIAGERETURN); Send_Command(TERMINAL_LINEFEEED); 
	Send_String("Taste <OK> - Beenden"); 
	Send_Command(MODE_END_INTERMEDIATE);     
	_delay_ms(1000);
	Send_Command(DRAW_CLEARSCREEN);
	
	Send_DRAW_Button("Farbauswahl", COORDINATES_BUTTON);
	
	for (;;)
	{
		c = Wait_For_Input;	
		
		if (c==SENDKEY_COORDINATES)
		{
			x = Wait_For_Input;	
			y = Wait_For_Input;
			
			
			
			if (mode==DRAW)
			{
				if (Point_in_Rect(x, y, COORDINATES_BUTTON) == 1)	// Knopf Links getroffen
				{
					Send_Command(DRAW_SELBUTTONFFRAME);
					Send_Double_Coordinates(COORDINATES_BUTTON);
					button_selected = 1;
					firstcoordinate = 1;
				}
				else
				{	
					if (button_selected==1)
					{
						Send_Command(DRAW_BUTTONFFRAME);
						Send_Double_Coordinates(COORDINATES_BUTTON);
						button_selected =0;
					}	
				
					if ((firstcoordinate == 0) || ((xalt==x) && (yalt=y)))
					{
						Send_Command(DRAW_LINE);
						Send_Double_Coordinates(xalt, yalt, x, y);

						xalt = x;
						yalt = y;
					}
					else
					{
						Send_Command(DRAW_SETPIXEL);
						Send_Coordinates(x, y);
						xalt = x;
						yalt = y;
						firstcoordinate = 0;
					}	
                }
				
			}
			else if (mode == PALETTE)
			{
				Send_Command(MODE_END_INTERMEDIATE);
				mode=DRAW;
				x = x/16;
				y = y/16;
				SelectedColor = 255-x-y*16;
				Send_Command(DRAW_COLOR);
				Send_Value(SelectedColor);
				firstcoordinate = 1;
				_delay_ms(200);
			}
		}
        else if (c==SENDKEY_STARTUP)
        {
            firstcoordinate = 1;
        }
        else if ((c==SENDKEY_TOUCH_END))
        {
            firstcoordinate = 1;
			if (button_selected==1)	// Knopf Links getroffen
			{
				Send_Command(DRAW_BUTTONFFRAME);
				Send_Double_Coordinates(COORDINATES_BUTTON);
				button_selected = 0;
				mode=PALETTE;
				Send_Command(DRAW_SELECTCOLOR);
			}
        }
		else if (c==SENDKEY_LEFT)	// Links-Taste bettigt
		{
			Send_Command(DRAW_CLEARSCREEN);
			button_selected = 0;
			Send_DRAW_Button("Farbauswahl", COORDINATES_BUTTON);
			Send_Command(DRAW_COLOR);
			Send_Value(SelectedColor);
		}
		else if (c==SENDKEY_RIGHT)	// Links-Taste bettigt
		{
			mode=PALETTE;
			Send_Command(DRAW_SELECTCOLOR);	
		}	
		else if (c==SENDKEY_OK)	
		{	
			// OK-Taste bettigt
			if (mode == PALETTE)
			{
				Send_Command(MODE_END_INTERMEDIATE);
				firstcoordinate = 1;
				mode=DRAW;
			}	
			else
				break;	
		}
	}
}




void Messwerterfassung(void)
{
	unsigned char c=0;
	double Value;
	unsigned int x;
	signed int delay;
	char String_Buffer[10];
	MessageBox(MESSAGEBOX_OK, "Messwerterfassung", "Bitte variable Eingangsspannung", "zwischen 0 und 5V an Eingang A0", "anschlieen.");
	delay = ShowKeyboard(10,500, NUMERIC_KEYBOARD, "Intervall (10-500ms) : ");
	if (delay != KEYBOARD_CANCELLED)
	{
		Send_Command(MODE_QUIT);  // Aktuellen Modus verlassen
		Send_Command(MODE_MESSWERTE);   // Messwerterfassung  
		Send_Command(MESSWERTE_TEXT);
		Send_String("Messwerterfassung, Intervall ");
		itoa(delay, String_Buffer,10);
		Send_String(String_Buffer);
		Send_String(" ms");
		
		Send_Command(MESSWERTE_WERTE);
	
		_delay_ms(500);
	
		DDRA  = 0x00;
		ADMUX = (0 << REFS1) | (1 << REFS0) ;
		ADCSRA = (1 << ADEN) | (1 << ADSC) | (1 << ADATE) | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0) | (0 << ADIE) ;
		ADCSRB = (0 << ADTS2) | (0 << ADTS1) | (0 << ADTS0) | (0 << ACME);
	
		for (;;)
		{
			Value = ADCW;
			Value = Value / 5.12 + 33;
			x = round(Value);
			Send_Value(x);
			_delay_ms(delay);
			c = Check_For_Input;
			if (c==SENDKEY_OK)	// OK-Taste bettigt
				break;
		}
		
		ADCSRA = 0x00;	
	}	
}




void Messinstrument(void)
{
	unsigned char c=0;
	double Value, max=-1, min=6;
	char String_Buffer[10];
	
	
	
	MessageBox(MESSAGEBOX_OK, "Messgert", "Bitte zu messende Spannung", "(0 bis max. 5V) an Eingang A0", "anschlieen.");
	
	Send_Command(MODE_QUIT);  // Aktuellen Modus verlassen
	Send_Command(MODE_TERMINAL);   // Messwerterfassung  
	Send_Command(TERMINAL_FONT_BIG);
	Send_Command(TERMINAL_SETCURSOR);
	Send_Coordinates(16, 10);
	Send_String("Spannungsmessung");
	Send_Command(TERMINAL_FONT_MEDIUM);
	Send_Command(TERMINAL_SETCURSOR);
	Send_Coordinates(16, 120);
	Send_String("Aktuelle Spannung:");
	
	Send_Command(TERMINAL_SETCURSOR);
	Send_Coordinates(16, 160);
	Send_String("Minimale Spannung:");
	
	Send_Command(TERMINAL_SETCURSOR);
	Send_Coordinates(16, 80);
	Send_String("Maximale Spannung:");

	_delay_ms(500);
	
	DDRA  = 0x00;
	ADMUX = (0 << REFS1) | (1 << REFS0) ;
	ADCSRA = (1 << ADEN) | (1 << ADSC) | (1 << ADATE) | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0) | (0 << ADIE) ;
	ADCSRB = (0 << ADTS2) | (0 << ADTS1) | (0 << ADTS0) | (0 << ACME);
	
	for (;;)
	{
		Value = ADCW;
		// Umrechnen in echte Spannung
		Value = Value * 4.883 / 1000;
		
		// Spannung ausgeben
		Send_Command(TERMINAL_SETCURSOR);
		Send_Coordinates(132, 120);
		dtostrf(Value, 4, 2, String_Buffer);
		Send_String(String_Buffer);
		Send_String(" Volt");
		
		if (Value < min)
		{
			min = Value;
			Send_Command(TERMINAL_TEXTCOLOR);
			Send_Command(16);	// grn
			Send_Command(TERMINAL_SETCURSOR);
			Send_Coordinates(132, 160);
			// Spannung ausgeben
			dtostrf(min, 4, 2, String_Buffer);
			Send_String(String_Buffer);
			Send_String(" Volt");
			Send_Command(TERMINAL_TEXTCOLOR);
			Send_Command(TextColor);
		}	
		else if (Value > max)
		{
			max = Value;
			Send_Command(TERMINAL_TEXTCOLOR);
			Send_Command(128);	// rot
			Send_Command(TERMINAL_SETCURSOR);
			Send_Coordinates(132, 80);
			// Spannung ausgeben
			dtostrf(max, 4, 2, String_Buffer);
			Send_String(String_Buffer);
			Send_String(" Volt");
			Send_Command(TERMINAL_TEXTCOLOR);
			Send_Command(TextColor);
		}	
		
		_delay_ms(100);
		c = Check_For_Input;
		if (c==SENDKEY_OK)	// OK-Taste bettigt
			break;
		else if (c== SENDKEY_LEFT)
		{
			// max- und min-Werte zurcksetzen
			max = Value - 0.01;
			min = Value + 0.01;
		}	
	}
	ADCSRA = 0x00;		
}



const char str_Getroffen[]	="Getroffen!  ";
const char str_Daneben[]	="Daneben...  ";
const char str_Verpasst[]	="Verpasst... ";




void Reaktionszeitspiel(void)
{
	int timer;
	unsigned char c=0;
	char String_Buffer[10];
	
	MessageBox(MESSAGEBOX_OK, "Reaktionszeitspiel", "Tippe auf die aufleuchtenden Rechtecke", "- aber nur, wenn sie blau sind...", "");
	Send_Command(MODE_QUIT);  // Aktuellen Modus verlassen
	Send_Command(MODE_8COLOR_LED);   // Farb-Demo    
	Send_Command(LED_8COLOR_BLUE);
	Send_Command(LED_NOBKCOLOR);
	_delay_ms(1000);
	Send_Command(LED_5_ON);
	timer = 0;
	// Zum ben
	for (;;)
	{
		c = Check_For_Input;
		_delay_ms(5);
		timer++;
		if ((c>=SENDKEY_TOUCH_START+16) && (c<=SENDKEY_TOUCH_START+31))
		{
			itoa(c, String_Buffer,10);
			Send_String(String_Buffer);
			_delay_ms(1000);
			if (c>=SENDKEY_TOUCH_START+24)
				c = c - SENDKEY_TOUCH_START - 24;
			else
				c = c - SENDKEY_TOUCH_START - 16;
			if (c==5) 
			{	
				if (timer > 20)
					Send_String(str_Verpasst);
				else
					Send_String(str_Getroffen);
			}
			else
				Send_String(str_Daneben);
			break;	
		}
		else if (timer > 400)
			break;	
	}	
	Send_Command(LED_8COLOR_CLEARSCREEN);
		
	// Jetzt richtig	
		
	for (;;)
	{
		Send_Command(LED_8COLOR_BLUE);
		Send_Command(LED_NOBKCOLOR);
		
		
		c = Check_For_Input;
		if (c==SENDKEY_OK)	// OK-Taste bettigt
			break;
	}
	//Send_String("Lauflicht-Demo"); 
/*
#define LED_8COLOR_BLUE				1		// Alle LEDs sind blau
#define LED_8COLOR_GREEN			2		// Alle LEDs sind gruen
#define LED_8COLOR_RED				3		// Alle LEDs sind rot
#define LED_8COLOR_YELLOW			4		// Alle LEDs sind gelb
#define LED_8COLOR_MIXED			5		// Je 2 LEDs sind blau, gruen, gelb, rot
#define LED_8COLOR_RAINBOW			6		// Regenbogen-Farbanordnung
#define LED_8COLOR_WHITE			7		// Alle LEDs sind wei

#define LED_8COLOR_CLEARSCREEN		8		// Lscht den Bildschirm

#define LED_1_OFF					9
#define LED_2_OFF					10
#define LED_3_OFF					11
#define LED_4_OFF					12
#define LED_5_OFF					13
#define LED_6_OFF					14
#define LED_7_OFF					15
#define LED_8_OFF					16
#define LED_1_ON					17
#define LED_2_ON					18
#define LED_3_ON					19
#define LED_4_ON					20
#define LED_5_ON					21
#define LED_6_ON					22
#define LED_7_ON					23
#define LED_8_ON					24
*/
	
}



const char str_Menu3_Caption[]="Programmauswahl (3/3)";
const char str_Menu3_1[]="Reaktionszeitspiel";
const char str_Menu3_2[]="Messgert";
const char str_Menu3_3[]="Texteingabe-Demo";
const char str_Menu3_4[]="Schieberegler-Demo";
const char str_Menu3_5[]="Weitere Programme...";


#define INPUT_BUFFER_SIZE 		20


unsigned char Menu3(void)
{
	unsigned char menu;
	char String_Buffer[INPUT_BUFFER_SIZE];
	int slider_value;
	for(;;)
	{
		menu = Show_Menu_Screen(str_Menu3_Caption, str_Menu3_1, str_Menu3_2, str_Menu3_3, str_Menu3_4, str_Menu3_5);	// Menauswahl 2
		switch(menu)
		{
			case MENUITEM1:
			{
				Reaktionszeitspiel();
				// MessageBox(MESSAGEBOX_OK, "Reaktionszeitspiel", "Noch nicht eingebaut", "", "");
				break;
			}	
			case MENUITEM2:
			{
				Messinstrument();
				break;
			}
			case MENUITEM3: 
			{
				if (ShowKeyboard(0, 0, ALPHANUMERIC_KEYBOARD, "") == 0)
					MessageBox(MESSAGEBOX_OK, "Ergebnis", "Eingegebene Daten", Value_Buffer, "");
				break;
			}
			case MENUITEM4:
			{
				slider_value = Slider("Schieberegler-Demo (min=0, max=1000)", 0, 400,1000);
				MessageBox(MESSAGEBOX_OK, "Ergebnis", "Schieberegler-Wert", itoa(slider_value, String_Buffer,10), "");
				break;
			}
			case MENUITEM5:
				break;
		}
		if (menu == MENUITEM5)	// Menu verlassen
			break;
	}
	return menu;
}



const char str_Menu2_Caption[]="Programmauswahl (2/3)";
const char str_Menu2_1[]="Taschenrechner";
const char str_Menu2_2[]="Tic Tac Toe";
const char str_Menu2_3[]="Touchscreen - Kalibrierung";
const char str_Menu2_4[]="Zeichnen auf dem Bildschirm";
const char str_Menu2_5[]="Weitere Programme...";




unsigned char Menu2(void)
{
	unsigned char menu;
	for(;;)
	{
		menu = Show_Menu_Screen(str_Menu2_Caption, str_Menu2_1, str_Menu2_2, str_Menu2_3, str_Menu2_4, str_Menu2_5);	// Menauswahl 2
		switch(menu)
		{
			case MENUITEM1:
			{
				Taschenrechner();
				break;
			}	
			case MENUITEM2:
			{
				// TicTacToe();
				MessageBox(MESSAGEBOX_OK, "Tic Tac Toe", "Noch nicht eingebaut", "", "");
				break;
			}
			case MENUITEM3: 
			{
				Kalibrierung();
				break;
			}
			case MENUITEM4:
			{
				Zeichnen();
				break;
			}
			case MENUITEM5:
				break;
		}
		if (menu == MENUITEM5)	// Menu verlassen
			break;
	}
	return menu;
}







const char str_Menu1_Caption[]="Programmauswahl (1/3)";
const char str_Menu1_1[]="Lauflicht";
const char str_Menu1_2[]="Ampelsteuerung";
const char str_Menu1_3[]="Messwerterfassung";
const char str_Menu1_4[]="Balkengrafik";
const char str_Menu1_5[]="Weitere Programme...";



unsigned char Menu1(void)
{
	unsigned char menu;
	for(;;)
	{
		menu = Show_Menu_Screen(str_Menu1_Caption, str_Menu1_1, str_Menu1_2, str_Menu1_3, str_Menu1_4, str_Menu1_5);	// Menauswahl 2
		switch(menu)
		{
			case MENUITEM1:
			{
				Lauflicht();
				break;
			}
			case MENUITEM2: 
			{
				Ampelsteuerung();
				break;
			}
			case MENUITEM3:
			{
				Messwerterfassung();
				break;
			}
			case MENUITEM4: 
			{
				Balkengrafik();
				break;
			}
			case MENUITEM5:
				break;
		}
		if (menu == MENUITEM5)	// Menu verlassen
			break;
	}
	return menu;
}




int main(void)
{
	unsigned char menu=0;

	sei();	

// UART Initialization
	uart_init(UART_BAUD_SELECT(UART_BAUD_RATE, XTAL_CPU));

	Show_Boot_Info();
	Show_Welcome_Screen();

//	Show_Serial_Debug_Screen();


    while(1)
	{
// loop forever timer does the job
		switch (menu)
		{
			case 0:
			{
				Menu1();
				menu++;
			}
			case 1:
			{
				Menu2();
				menu++;
			}
			case 2:
			{
				Menu3();
				menu=0;
			}
		}
    }
	return(0);
}





	
    