/****************************************************************************
*
* Atmel Corporation
*
* File              : main.c
* Compiler          : WinAVR V4.11
* Revision          : $Revision: 0.99 $
* Date              : $Date: Tuesday, April 20, 2013 $
* Original by       : $Author: Jens Rotter, Atmel 2006 $
* Modification      : Frank Brennecke
*
* Support mail      : avr@atmel.com
*
* Supported devices : ATmega88
*
* AppNote           : ?
* 
* Description       : Touchscreen Controller 4wire (Lowlevel/Rtouch Measurement) and 5wire (Lowlevel Measurement)
*
* Modification      : Controller transmits an 8x6 area field instead of coordinates
*                     Supports three buttons for a menu selection
*                     Supports special mode for calibration (press three buttons at once)
*
****************************************************************************/





/****************************************************************************
  Libraries
****************************************************************************/

#include <avr/io.h> 
#include <stdlib.h> 
#include <avr/eeprom.h>
#include <avr/interrupt.h> 
#include "Touchscreen.c"
#include "usart.c"
#include <util/delay.h>
#include "SimpleKeyboard.c"
#include "SendKeys.h"



/****************************************************************************
  Global definitions
****************************************************************************/




// Set Interface
#define UART_INTERFACE

// Flags to interpret
#define FLAG_REGISTER_MASK 255

// Additional defines
#define TOUCHSCREEN_TOLERANCE 		4
#define STRING_BUFFER_SIZE 5



// Global variables

// Kalibrierungsdaten


/****************************************************************************
  Function definitions
****************************************************************************/


unsigned char Recognize_Event(void);
void Send_over_UART(void);
void Send_Key_over_UART(void);

/****************************************************************************
  Global variable
****************************************************************************/
// Touchscreen Interface
extern Interface Touchscreen_Data;
Interface Buffer_Touchscreen_Data;

// abgespeicherte Werte
// diese Werte sind ein wenig zu klein, damit alle Elemente sicher erreicht werden können
uint16_t X_START_EE EEMEM = 130;                     // linke Seite liefert bei Berührung diesen Wert
uint16_t X_END_EE EEMEM = 880;                       // Rechte Seite liefert bei Berührung diesen Wert
uint16_t Y_START_EE EEMEM = 220;                     // Oberer Rand liefert bei Berührung diesen Wert
uint16_t Y_END_EE EEMEM = 800;                       // Unterer Rand liefert bei Berührung diesen Wert

// Kalibrierungsdaten
unsigned int X_START = 130;                     // linke Seite liefert bei Berührung diesen Wert
unsigned int X_END = 880;                       // Rechte Seite liefert bei Berührung diesen Wert
unsigned int Y_START = 220;                     // Oberer Rand liefert bei Berührung diesen Wert
unsigned int Y_END = 800;                       // Unterer Rand liefert bei Berührung diesen Wert

// häufiger bebnötigte Variablen
unsigned int Pixels_Per_Field_X;
unsigned int Pixels_Per_Field_Y;

// Betriebsmodus
unsigned int User_Mode = MODE_FIELDS;

// Variablen zur Feld- und Koordinatenübermittlung
unsigned int last_field=0, last_x=0, last_y=0;

// Variablen zur Konfiguration des Touchscreens
unsigned char xlt, ylt, xlb, ylb;
unsigned char xrt, yrt, xrb, yrb;
unsigned char Configuration_Corner = 0;
unsigned char Send_Data_Corner = 0;

/****************************************************************************
  main loop
****************************************************************************/

int main(void)
{  
    // PORT initialisation
	DDRD = 0x02;		// Port D: Input bei D6 und D7 durch Grafikcontroller, Output D1  
	PORTD = 0x00;		// Pull-up-Widerstände insbesondere D6 und D7 abschalten 
						// wichtig, weil ansonsten die Umschaltung der Modi ewig dauert
	
	// USART Initialization
    USART_Init(MY_UBRR);
	
	// Read from EEPROM	
	X_START = eeprom_read_word (&X_START_EE);
	X_END = eeprom_read_word (&X_END_EE);
	Y_START = eeprom_read_word (&Y_START_EE);
	Y_END = eeprom_read_word (&Y_END_EE);
        
    // Touchscreen Initialization
    Touchscreen_Init();
    Pixels_Per_Field_X = (X_END - X_START) / (X_FIELDS + 1);
    Pixels_Per_Field_Y = (Y_END - Y_START) / (Y_FIELDS + 1);
    last_field = 0;
    
    // Keyboard Initialization
    Keyboard_Init();
    
    // Enable global Interrupts
    sei();
    
    // Infinite Loop
	for (;;)
	{
        if (Recognize_Event())
            Send_over_UART();
            
        if (GetKeys() != 0)
            Send_Key_over_UART();
		
	} // for
}

unsigned char Recognize_Event(void)
{
    cli();
    // Read Data
    Buffer_Touchscreen_Data = Touchscreen_Data;
    // Mask Flag_Register
    Buffer_Touchscreen_Data.Flag_Register &= FLAG_REGISTER_MASK;
    // Clear Joblist
    Touchscreen_Data.Flag_Register ^= Buffer_Touchscreen_Data.Flag_Register;
       
    sei();
    return Buffer_Touchscreen_Data.Flag_Register;
}


void Send_Key_over_UART(void)
{
    switch (Keys)
	{
		case KEY_LEFT:
		{	
			uart_putc(SENDKEY_LEFT);
			break;
		}	
		case KEY_RIGHT:
		{
			uart_putc(SENDKEY_RIGHT);
			break;
		}	
		case KEY_OK:
		{
			uart_putc(SENDKEY_OK);	
			break;
		}	
	}
	_delay_ms(100);	// Entprellen

}




char *_utoa(char* buffer, const unsigned int size, unsigned int value)
{
  char *p;
 
  // p points to last char
  p = buffer+size-1;
  
  // Set terminating Zero 
  *p='\0';
  
  do 
  {
    *--p = value%10 + '0';
    value = value/10;
  } while (value!=0 && p>buffer);
  
  return p;
}



void Send_over_UART(void)
{
	unsigned char c, curr_value;
//	char String_Buffer[STRING_BUFFER_SIZE]; 
// prüfen, welche Information gesendet werden soll
	
	User_Mode = PIND & MODE_BITMASK;
		
	if ((User_Mode == MODE_CALIBRATION) && (Configuration_Corner == 0))
	{
		Configuration_Corner = 1;
	}

	  
    if(TestBit(Buffer_Touchscreen_Data.Flag_Register, FLAG_REGISTER_OVERFLOW))
    {
        if (User_Mode != MODE_SILENT)    
			uart_putc(SENDKEY_ERROR);	// Overflow: X/Y-Coordinates were overwritten
    }
    else
    {
        if(TestBit(Buffer_Touchscreen_Data.Flag_Register, FLAG_REGISTER_START))
        {
        // Start of Programm --> e.g. collect Calibration Points for higher Level Software
			uart_putc(SENDKEY_STARTUP);
        }
              
        if(TestBit(Buffer_Touchscreen_Data.Flag_Register, FLAG_REGISTER_SLEEP))
        {
            // Nothing to do... --> e.g. Powerdown
            // uart_putc(SENDKEY_SLEEP);

            // Power down
            //SMCR = (2<<SM0)|(1<<SE);
            //__sleep(); 
        }
		
		if(TestBit(Buffer_Touchscreen_Data.Flag_Register, FLAG_REGISTER_TOUCH))
        {
            
            // Touch detected
			uart_putc(SENDKEY_TOUCH_START);
		}
                      
        if(TestBit(Buffer_Touchscreen_Data.Flag_Register, FLAG_REGISTER_COORDINATES))
        {
            
            // Touch detected
            if (User_Mode == MODE_CALIBRATION)
            {
				switch (Configuration_Corner)
				{
					case 1: // links oben
					{
						xlt = Buffer_Touchscreen_Data.x_pos / 8;
						ylt = Buffer_Touchscreen_Data.y_pos / 8;
						Send_Data_Corner = 1;
						break;
					}
					case 2:
					{
						xrt = Buffer_Touchscreen_Data.x_pos / 8;
						yrt = Buffer_Touchscreen_Data.y_pos;
						Send_Data_Corner = 2;
						break;
					}
					case 3: // links unten
					{
						xlb = Buffer_Touchscreen_Data.x_pos / 8;
						ylb = Buffer_Touchscreen_Data.y_pos / 8;
						Send_Data_Corner = 3;
						break;
					}
					case 4:
					{
						xrb = Buffer_Touchscreen_Data.x_pos / 8;
						yrb = Buffer_Touchscreen_Data.y_pos / 8;
						Send_Data_Corner = 4;
						break;
					}
				}
				
            }
            else if (User_Mode == MODE_FIELDS)
            {
                signed int x1,y1;
				x1 = (Buffer_Touchscreen_Data.x_pos - X_START) / Pixels_Per_Field_X;
                if (x1 < 0)               // unsaubere Kalibrierung
                    x1 = 0;
                else if (x1 > X_FIELDS)   // unsaubere Kalibrierung
                    x1 = X_FIELDS;
                y1 = (Buffer_Touchscreen_Data.y_pos - Y_START) / Pixels_Per_Field_Y;
				y1 =  Y_FIELDS - y1;
                if (y1 < 0)               // unsaubere Kalibrierung
                    y1 = 0;
                else if (y1 > Y_FIELDS)   // unsaubere Kalibrierung
                    y1 = Y_FIELDS;
                curr_value = y1 * (X_FIELDS+1) + x1;  // berechnen des getroffenen Feldes
				if (curr_value != last_field)	// nicht mehrfach senden
				{
					last_field = curr_value;
					uart_putc(last_field + SENDKEY_TOUCH_START);
				}	
                
            }
			else if (User_Mode == MODE_COORDINATES)
			{
				signed long x1,y1;
				x1 = Buffer_Touchscreen_Data.x_pos - X_START;
				x1 = x1*800 / (X_END-X_START);
				x1 = x1 * X_COORDINATES / 800;
                if (x1 < 0)               // unsaubere Kalibrierung
                    x1 = 0;
                else if (x1 > X_COORDINATES)   // unsaubere Kalibrierung
                    x1 = X_COORDINATES;
                y1 = Buffer_Touchscreen_Data.y_pos - Y_START;
				y1 = y1* 800 / (Y_END-Y_START);
				y1 = y1 * Y_COORDINATES  / 800;
				y1 = 251 - y1; 
				if (y1 < 0)               // unsaubere Kalibrierung
                    y1 = 0;
                else if (y1 > Y_COORDINATES)   // unsaubere Kalibrierung
                    y1 = Y_COORDINATES;
				if ((x1 !=  last_x) || (y1 !=  last_y))	// nicht doppelt senden
				{
					uart_putc(SENDKEY_COORDINATES);
					c = x1;	last_x = x1;
					uart_putc(c);	
					c = y1;	last_y = y1;	
					uart_putc(c);
				}	
			}
        }      
		
        if (TestBit(Buffer_Touchscreen_Data.Flag_Register, FLAG_REGISTER_END))
        {
            // dragging ignored, same field as touch down
            // nur senden, wenn Auswahl von Feldern gewünscht sind
            if (User_Mode == MODE_FIELDS)
			{
                uart_putc(last_field + SENDKEY_TOUCH_END);
				last_field = 0;
			}
			else if (User_Mode == MODE_COORDINATES)
			{
				uart_putc(SENDKEY_TOUCH_END);
			}
			else if (User_Mode == MODE_CALIBRATION)
			{
				switch (Send_Data_Corner)
				{
					case 1:
					{
						// prüfe links oben
						if ((xlt > 24) || (ylt < 100))
						{
							uart_putc(SENDKEY_RESTART_CONFIG_POSITION);
							Configuration_Corner = 0;
						}
						else 
						{
							uart_putc(SENDKEY_CONFIG_2);
							Configuration_Corner = 2;
						}	
						// uart_putc(1);
						break;
					}
						
					case 2: // oben komplett - diese Ecke ist die ungenaueste
					{ 
						
						// prüfe rechts oben
						if ((xrt < 90) || (yrt < 90))
						{
							uart_putc(SENDKEY_RESTART_CONFIG_POSITION);
							Configuration_Corner = 0;
						}
						
						else if (abs(ylt-yrt) < 3*TOUCHSCREEN_TOLERANCE)  // Toleranzschwelle deutlich erhöhen, sonst nervt es
						{
							uart_putc(SENDKEY_CONFIG_3);
							Configuration_Corner = 3;
						}	
						else
						{
							uart_putc(SENDKEY_RESTART_CONFIG_Y);
							Configuration_Corner = 0;
						}
						// uart_putc(2);	
						break;	
					}	
					case 3: // links komplett
					{ 
						
						// prüfe links unten
						if ((xlb > 24) || (ylb > 24))
						{
							uart_putc(SENDKEY_RESTART_CONFIG_POSITION);
							Configuration_Corner = 0;
						}
						else if (abs(xlt-xlb) < TOUCHSCREEN_TOLERANCE)
						{
							uart_putc(SENDKEY_CONFIG_4);
							Configuration_Corner = 4;
						}	
						else
						{
							uart_putc(SENDKEY_RESTART_CONFIG_X);
							Configuration_Corner = 0;
						}	
						// uart_putc(3);	
						break;	
					}	
					case 4:	// rechts und unten komplett
					{
						// prüfe rechts unten
						if ((xrb < 100) || (yrb > 24))
						{
							uart_putc(SENDKEY_RESTART_CONFIG_POSITION);
							Configuration_Corner = 0;
						}
						else if (abs(yrb-ylb) < TOUCHSCREEN_TOLERANCE)
						{
							if (abs(xrt-xrb) < TOUCHSCREEN_TOLERANCE)
							{
								uart_putc(SENDKEY_CONFIG_COMPLETE);
								
								X_END = (xrt + xrb)*4;
								X_START = (xlt + xlb)*4;
								Pixels_Per_Field_X = (X_END - X_START) / (X_FIELDS + 1);
								Y_END = (ylt + yrt)*4;
								Y_START = (ylb + yrb)*4;
								Pixels_Per_Field_Y = (Y_END - Y_START) / (Y_FIELDS + 1);
								
								// Write to EEPROM
								eeprom_write_word(&X_START_EE, X_START);
								eeprom_write_word(&X_END_EE, X_END);
								eeprom_write_word(&Y_START_EE, Y_START);
								eeprom_write_word(&Y_END_EE, Y_END);
								
							}	
							else
								uart_putc(SENDKEY_RESTART_CONFIG_X);
						}	
						else	
							uart_putc(SENDKEY_RESTART_CONFIG_Y);
						Configuration_Corner = 0;
						// uart_putc(4);
						break;	 
					}
				}
				Send_Data_Corner = 0;
				
			}
			
        }
	}
}


