/****************************************************************************
 *
 *
 * File              : mclib.c
 * Compiler          : WinAVR 4.11
 * Revision          : $Revision: 0.99 $
 * Date              : $Date: Wednesday, May 15, 2013 $
 * Updated by        : $Author: Frank Brennecke $
 *
 * 
 * Supported devices : ATmega88, ATMega644
 * Description       : Interface definitions using Touchscreen ansd Graphics Library
 *
 ****************************************************************************/


#define COORDINATES_BUTTON_LEFT 	3,190,93,247
#define COORDINATES_BUTTON_OK		98,190,158,247
#define COORDINATES_BUTTON_RIGHT	163,190,253,247


#define Send_Command			uart_putc	
#define Send_Value				uart_putc
#define Send_String				uart_puts
#define	Wait_For_Input 			uart_getchar()
#define	Check_For_Input 		uart_getc()


void Send_Coordinates(unsigned char x, unsigned char y)
{
	Send_Command(x);
	Send_Command(y);
}


void Send_Double_Coordinates(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2)
{
	Send_Command(x1);
	Send_Command(y1);
	Send_Command(x2);
	Send_Command(y2);
}



void Send_Terminal_String(unsigned char x, unsigned char y, unsigned char delete_line, const char *s)
{
	char i;
	if (delete_line != 0)
	{
		Send_Command(TERMINAL_SETCURSOR); 
		Send_Coordinates(0, y);
		for (i=0; i<42; i++)
			Send_Command(32);
	}	
	Send_Command(TERMINAL_SETCURSOR); 
	Send_Coordinates(x, y);
	Send_String(s); 
}



char Point_in_Rect(unsigned char x, unsigned char y ,unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2)
{
	if ((x>x1) && (x<=x2) && (y>y1) && (y<=y2))
		return 1;
	else
		return 0;
}


/*==============================================*/
/*==== Menu-Funktionen	====================*/



#define MENUITEM1	1
#define MENUITEM2	2
#define MENUITEM3	3
#define MENUITEM4	4
#define MENUITEM5	5




int Show_Menu_Screen(const char *caption, const char *menu1, const char *menu2, const char *menu3, const char *menu4, const char *menu5)
{
	unsigned char c=0, selected=1;
	
	Send_Command(MODE_QUIT);  // Aktuellen Modus verlassen
    Send_Command(MODE_MENU);   // Menauswahl anzeigen
	
	_delay_ms(200);
	
	Send_Command(MENU_TEXT_LINE1);	// Zeile 1
	Send_String(caption); 
	Send_Command(MENU_TEXT_LINE2);	// Zeile 2
	Send_String(menu1); 
	Send_Command(MENU_TEXT_LINE3);	// Zeile 3
	Send_String(menu2); 
	Send_Command(MENU_TEXT_LINE4);	// Zeile 4
	Send_String(menu3); 
	Send_Command(MENU_TEXT_LINE5);	// Zeile 5
	Send_String(menu4); 
	Send_Command(MENU_TEXT_LINE6);	// Zeile 6
	Send_String(menu5); 
	Send_Command(MENU_LINE2_SELECTED);	// Selektiere Zeile 2
	Send_Command(MENU_DRAWFRAME);  // neuzeichnen
	for(;;)
	{
		c = Wait_For_Input;
		if (c==SENDKEY_OK)	// OK-Taste bettigt
		{
			break;
		}
		else if (c==SENDKEY_LEFT)
		{
			selected--;
			if (selected==MENUITEM1-1)
				selected = MENUITEM5;
			Send_Command(MENU_LINE1_SELECTED + selected);	// Selektiere aktuelle Zeile
			Send_Command(MENU_DRAWFRAME);	
		}
		else if (c==SENDKEY_RIGHT)
		{
			selected++;
			if (selected==MENUITEM5+1)
				selected = MENUITEM1;
			Send_Command(MENU_LINE1_SELECTED + selected);	// Selektiere aktuelle Zeile
			Send_Command(MENU_DRAWFRAME);	
		}
		else if ((c>=SENDKEY_TOUCH_START) && (c<SENDKEY_TOUCH_START+48))
		{
			c = c/8;
			if ((c>=1) && (c<6))
			{
				selected = c;
				Send_Command(MENU_LINE1_SELECTED + selected);	// Selektiere aktuelle Zeile
				Send_Command(MENU_DRAWFRAME);	
			}	
		}	
		else if ((c>=SENDKEY_TOUCH_END) && (c<SENDKEY_TOUCH_END+48))
		{
			c = (c-SENDKEY_TOUCH_END+1)/8;
			if ((c>=1) && (c<6))
			{
				selected = c;
				Send_Command(MENU_LINE1_SELECTED + selected);	// Selektiere aktuelle Zeile
				Send_Command(MENU_DRAWFRAME);	
				_delay_ms(200);
				break;
			}	
		}	
		
	}
	return selected;
}





/*==============================================*/
/*==== Keyboard-Funktionen	====================*/


#define MAXBUTTONS 				38
#define MAXNUMERICBUTTONS		10
#define KEYBOARD_MINMAXERROR	-1
#define KEYBOARD_CANCELLED		-2  // Abbrechen-Knopf gedrckt
#define NUMERIC_KEYBOARD		1
#define ALPHANUMERIC_KEYBOARD		0
#define OWNERDRAW_KEYBOARD		3
#define STRING_BUFFER_SIZE 5




#define INPUT_BUFFER_SIZE 41

char Value_Buffer[INPUT_BUFFER_SIZE];

const char str_InputError[]="Fehler...";




void WriteKeyboardCaption(const char *value1, char separator, const char *value2)
{
	unsigned char NumChars;
	Send_Command(KEYBOARD_RESET);
	NumChars = 0;
	while (*value1) 
	{
		Send_Command(KEYBOARD_TEXT + *value1++);
		NumChars++;
	}	
	Send_Command(KEYBOARD_TEXT + 32);	// Leerzeichen
	Send_Command(KEYBOARD_TEXT + separator);	// Leerzeichen
	Send_Command(KEYBOARD_TEXT + 32);	// Leerzeichen
	while (*value2) 
	{
		Send_Command(KEYBOARD_TEXT + *value2++);
		NumChars++;
	}	
	while (NumChars++ < 40)
		Send_Command(KEYBOARD_TEXT + 32);
}



void DrawButton(unsigned char c, unsigned char State)
{
	if (c < MAXBUTTONS)
	{
		if (State == KEYBOARD_RELEASE)
		{
			if (c + KEYBOARD_RELEASE == MODE_QUIT)
				Send_Command(KEYBOARD_RELEASE31);
			else
				Send_Command(KEYBOARD_RELEASE + c);
		}
		else
			Send_Command(State + c);
	}		
}


unsigned char CalculateTouchButton(unsigned char c, unsigned char offset, char Keyboard_Mode)
{
	if ((c == offset + 44) || (c == offset + 45))	// OK	
		return KEYBOARD_OK;
	else if ((c == offset + 46) || (c == offset + 47))  // Cancel
		return KEYBOARD_CANCEL;
	else 
	{
		c = c - offset - 8;
		if (Keyboard_Mode == NUMERIC_KEYBOARD)
		{
			// codes grer 9 sind nicht erlaubt
			if (c > 9)
				c = MAXBUTTONS;
		} 
		else if (c >= MAXBUTTONS)
			c = MAXBUTTONS;
		return c;
	}	
}




char Check_Keyboard_Keys(unsigned char c, unsigned char selected, unsigned char Keyboard_Mode)
{
	if ((c >= SENDKEY_TOUCH_END+8) && (c <= SENDKEY_TOUCH_END + 47))	// LOsgelassener Touch-Knopf
	{
		DrawButton(selected, KEYBOARD_RELEASE);
		selected = CalculateTouchButton(c, SENDKEY_TOUCH_END, Keyboard_Mode);	// Sicherheitshalber neu berechnen	
		DrawButton(selected, KEYBOARD_RELEASE);
		_delay_ms(200);	// damit man den losgelassenen Knopf auch sieht
	}
	else if ((c >= SENDKEY_TOUCH_START+8) && (c <= SENDKEY_TOUCH_START + 47))  // Gedrckter Touch-Knopf
	{
		DrawButton(selected, KEYBOARD_RELEASE);	
		selected = CalculateTouchButton(c, SENDKEY_TOUCH_START, Keyboard_Mode);
		DrawButton(selected, KEYBOARD_PRESSED);
	}	
	else if (c==SENDKEY_LEFT)
	{
		DrawButton(selected, KEYBOARD_RELEASE);
		if (selected == 0)
			selected = MAXBUTTONS-1;
		else
		{
			if (Keyboard_Mode != NUMERIC_KEYBOARD)
				selected--;
			else if (selected == KEYBOARD_OK)
				selected = MAXNUMERICBUTTONS-1;
			else
				selected--;
		}	
		DrawButton(selected, KEYBOARD_SELECT);
	}
	else if (c==SENDKEY_RIGHT)
	{
		DrawButton(selected, KEYBOARD_RELEASE);
		if (Keyboard_Mode != NUMERIC_KEYBOARD)
			selected++;	
		else if (selected ==  MAXNUMERICBUTTONS-1)
			selected = KEYBOARD_OK;
		else	
			selected++;
		if (selected >= MAXBUTTONS)
			selected = 0;
		DrawButton(selected, KEYBOARD_SELECT);	
	}
	return selected;
}


signed int ShowKeyboard(int Min, int Max, char Keyboard_Mode, const char *caption)
{
	unsigned char c=0, selected=MAXBUTTONS, CurrentChar=0;
	signed int value;
	
	if ((Min >= Max) && (Keyboard_Mode == NUMERIC_KEYBOARD))
		value = KEYBOARD_MINMAXERROR;
	else	
	{
		value = 0;
		Value_Buffer[CurrentChar] = 0;
		Send_Command(MODE_QUIT);
		if (Keyboard_Mode == NUMERIC_KEYBOARD)
			Send_Command(MODE_KEYBOARD_NUMERIC);
		else
			Send_Command(MODE_KEYBOARD_ALPHA);
		WriteKeyboardCaption(caption, 32, "");	
		for(;;)
		{
			c = Wait_For_Input;

// erst die Touch-Werte auswerten und ggf. den Wert fr c ndern
			selected = Check_Keyboard_Keys(c, selected, Keyboard_Mode);
			if ((c >= SENDKEY_TOUCH_END+8) && (c < SENDKEY_TOUCH_END + 47))	// LOsgelassener Touch-Knopf
				c=SENDKEY_OK;
			
			if (c==SENDKEY_OK)	// OK-Taste bettigt
			{
				if (selected == KEYBOARD_OK)
				{
					if (Keyboard_Mode == ALPHANUMERIC_KEYBOARD)
					{
						if (CurrentChar > 0)	// Eingabe vorhanden und bereits geprft
							break;
					}	
					else if ((value <= Max) && (value >= Min))  // Eingabe auf bergabewerte prfen
						break;
					value = 0;
					CurrentChar = 0;
					Value_Buffer[CurrentChar] = 0;
					WriteKeyboardCaption(caption, 32, str_InputError);
					_delay_ms(500);		
				}
				else if (selected == KEYBOARD_CANCEL)	
				{
					value = KEYBOARD_CANCELLED;
					break;
				}				
				if (Keyboard_Mode == NUMERIC_KEYBOARD)
				{
					if ((Min >= 0) && (Max < 10))	// nur eine Stelle zugelassen, kann gleich ersetzt werden
					{
						value = selected;
						if (value > Max)
							value = Max;
						else if (value < Min)
							value = Min;
						CurrentChar = 0;
						Value_Buffer[CurrentChar++] = KEYBOARD_0  + value;
						Value_Buffer[CurrentChar] = 0;
					}
					else if (CurrentChar < INPUT_BUFFER_SIZE)
					{
						value = value*10 + selected;
						Value_Buffer[CurrentChar++] = KEYBOARD_0  + selected;
						Value_Buffer[CurrentChar] = 0;
					}
				}
				else if (CurrentChar < INPUT_BUFFER_SIZE)
				{
					if (selected < MAXNUMERICBUTTONS)
						Value_Buffer[CurrentChar++] = KEYBOARD_0  + selected;
					else if (selected < KEYBOARD_OK)	
						Value_Buffer[CurrentChar++] = KEYBOARD_A  + selected - 10;
					Value_Buffer[CurrentChar] = 0;
				}	
				WriteKeyboardCaption(caption, 32, Value_Buffer);	
			}	
			
			
			
		}
	}
	return value;
}



/*


char *FormatValue(char* buffer, const unsigned int size, unsigned int value)
{
	char *p, char i=0, j=0;
 
	return buffer;
 
  // p points to last char
	p = buffer;
  
	while (i < size) 
	{
		if (p[i] != 0)
			p++
		else
			break;	// i zeigt nun auf das Ende des Strings
		i++;	
	}
	
	
	
	
	
	while (i < size) 
	{
		if p[i] <> 0
			p++
		else
			break;	// i zeigt nun auf das Ende des Strings
		i++;	
	
	}
	
	
	
  // Set terminating Zero 
	*p='\0';
  
	do 
	{
		*--p = value%10 + '0';
		value = value/10;
	} 
	while (value!=0 && p>buffer);
  
	
}

*/

#define OKButtonTouched 	(c==SENDKEY_TOUCH_START+43) || (c==SENDKEY_TOUCH_START+44)
#define LeftButtonTouched	(c==SENDKEY_TOUCH_START+40) || (c==SENDKEY_TOUCH_START+41) || (c==SENDKEY_TOUCH_START+42)
#define RightButtonTouched	(c==SENDKEY_TOUCH_START+45) || (c==SENDKEY_TOUCH_START+46) || (c==SENDKEY_TOUCH_START+47)


#define OKButtonPressed 	(c==SENDKEY_TOUCH_END+43) || (c==SENDKEY_TOUCH_END+44)
#define LeftButtonPressed	(c==SENDKEY_TOUCH_END+40) || (c==SENDKEY_TOUCH_END+41) || (c==SENDKEY_TOUCH_END+42)
#define RightButtonPressed	(c==SENDKEY_TOUCH_END+45) || (c==SENDKEY_TOUCH_END+46) || (c==SENDKEY_TOUCH_END+47)

#define MR_OK			0
#define MR_CANCEL		1
#define MR_YES			2
#define MR_NO			3
#define MR_NEXT			4
#define MR_BACK			5
#define MR_ERROR		255



unsigned char MessageBox(char mode, const char *caption, const char *line1, const char *line2, const char *line3)
{
	unsigned char c;
	
	
	Send_Command(MODE_QUIT);  // Aktuellen Modus verlassen
    Send_Command(MODE_MESSAGEBOX);   // 
		
	// Mode prfen
	if ((mode >= MESSAGEBOX_BACK_NEXT) && (mode <= MESSAGEBOX_OK))
	{
		Send_Command(mode);	
		Send_Command(MESSAGEBOX_LINE1);	// Zeile 1
		Send_String(caption); 
		Send_Command(MESSAGEBOX_LINE2);	// Zeile 2
		Send_String(line1); 
		Send_Command(MESSAGEBOX_LINE3);	// Zeile 3
		Send_String(line2); 
		Send_Command(MESSAGEBOX_LINE4);	// Zeile 4
		Send_String(line3); 

		for(;;)
		{
			c = Wait_For_Input;
			if (LeftButtonTouched)
			{
				if (mode != MESSAGEBOX_OK)
					Send_Command(MESSAGEBOX_SELECTBTN1);
			}	
			else if (OKButtonTouched)
			{
				if ((mode == MESSAGEBOX_BACK_OK_NEXT) || (mode == MESSAGEBOX_OK))
					Send_Command(MESSAGEBOX_SELECTBTN2);
			}	
			else if (RightButtonTouched)
			{
				if (mode != MESSAGEBOX_OK)
					Send_Command(MESSAGEBOX_SELECTBTN3);	
			}
			else
				Send_Command(mode);	// zeichnet die Knpfe im ungedrckten Zustand neu
				
			// jetzt Prfung, ob Knpfe ausgelst wurden	
			if ((c==SENDKEY_OK) || (OKButtonPressed))
			{
				if ((mode == MESSAGEBOX_BACK_OK_NEXT) || (mode == MESSAGEBOX_OK) || (mode == MESSAGEBOX_OK_CANCEL))
				{
					return MR_OK;
					break;
				}	
			}
			else if ((c==SENDKEY_LEFT) || (LeftButtonPressed))
			{
				if ((mode == MESSAGEBOX_BACK_NEXT) || (mode == MESSAGEBOX_BACK_OK_NEXT))
				{
					return MR_BACK;
					break;
				}
				else if (mode == MESSAGEBOX_OK_CANCEL)
				{
					return MR_OK;
					break;
				}
				else if (mode == MESSAGEBOX_YES_NO)
				{
					return MR_YES;
					break;
				}
			}
			else if ((c==SENDKEY_RIGHT) || (RightButtonPressed))
			{
				if ((mode == MESSAGEBOX_BACK_NEXT) || (mode == MESSAGEBOX_BACK_OK_NEXT))
				{
					return MR_NEXT;
					break;
				}
				else if (mode == MESSAGEBOX_OK_CANCEL)
				{
					return MR_CANCEL;
					break;
				}
				else if (mode == MESSAGEBOX_YES_NO)
				{
					return MR_NO;
					break;
				}
			}	
		}
	}	
	else
		return MR_ERROR;
}



#define SLIDER_RANGE 200

int Slider(const char *caption, int min, int value, int max)
{
	unsigned char slider_value, x, y, c;
	float fvalue;
	char LeftButton=0, RightButton=0, OKButton=0; 
	
	if (min >= max)
		return value;
	else 
	{		
		Send_Command(MODE_QUIT);  // Aktuellen Modus verlassen
		Send_Command(MODE_SLIDER);   // 
		
		slider_value = value * SLIDER_RANGE / (max - min);
		if (slider_value > 199)
			slider_value = 199;
		
		Send_Command(MODE_SLIDER);
		Send_String(caption);
		Send_Command(SLIDER_VALUE);
		Send_Command(slider_value);
		
		for(;;)
		{
			c = Wait_For_Input;
			
			if (c==SENDKEY_COORDINATES)
			{
				x = Wait_For_Input;
				y = Wait_For_Input;
				// Prfe, welche Taste getroffen wurde
				LeftButton=0;
				RightButton=0;
				OKButton=0;
				if ((y > 210) && (x >= 96) && (x < 160))	// untere Ebene getroffen
					OKButton = 2;
				else if ((y>=111) && (y <152))	// Slider getroffen	
				{
					if (x < 25)
						LeftButton = 2;
					else if (x >= 225)
						RightButton = 2;
					else	
					{
						slider_value = x - 25;
						Send_Command(SLIDER_VALUE);
						Send_Command(slider_value);
					}	
				}
			}
			else
				Send_Command(SLIDER_REDRAW);
			
			// Knpfe zeichnen
			if (LeftButton == 2)
				Send_Command(SLIDER_BTN_BACK_PRESSED);
			else if (OKButton == 2)
				Send_Command(SLIDER_BTN_OK_PRESSED);
			else if (RightButton == 2)
				Send_Command(SLIDER_BTN_NEXT_PRESSED);	
			else
				Send_Command(SLIDER_REDRAW);	// zeichnet die Knpfe im ungedrckten Zustand neu
				
			if (c==SENDKEY_TOUCH_END)	
			{
				if (LeftButton == 2)
					LeftButton--;
				if (RightButton == 2)
					RightButton--;
				if (OKButton == 2)
					OKButton--;
			}
			
			
			if ((c==SENDKEY_OK) || (OKButton == 1))
			{
				fvalue = slider_value;
				return min + fvalue * (max - min) / 200;
				OKButton = 0;
			}
			else if ((c==SENDKEY_LEFT) || (LeftButton == 1))
			{
				if (slider_value > 0)
				{
					Send_Command(SLIDER_VALUE);
					Send_Command(--slider_value);
					LeftButton = 0;
				}	
			}
			else if ((c==SENDKEY_RIGHT) || (RightButton == 1))
			{
				if (slider_value < 199)
				{
					Send_Command(SLIDER_VALUE);
					Send_Command(++slider_value);
					RightButton = 0;
				}	
			}
		}
	}	
}


void Send_DRAW_Button(const char *caption, unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2)
{
	int iValue;
	char *buffer;
	
	Send_Command(DRAW_COLOR);
	Send_Value(WindowBkColor);	// Zeichenfarbe fr Grafikausgabe
	Send_Command(DRAW_FILLRECT);
	Send_Double_Coordinates(x1-1, y1-1, x2+1, y2+2);
	Send_Command(DRAW_COLOR);
	Send_Value(255);	// Hintergrundfarbe fr Grafikausgabe
	Send_Command(DRAW_BUTTONFFRAME);
	Send_Double_Coordinates(x1, y1, x2, y2);
	Send_Command(DRAW_SWITCHTERMINAL);  // In Terminal-Modus umschalten
	if (y2-y1 < 25)
		Send_Command(TERMINAL_FONT_SMALL);
	else	
		Send_Command(TERMINAL_FONT_MEDIUM);
	Send_Command(TERMINAL_BKCOLOR);
	Send_Value(WindowBkColor);	// Hintergrundfarbe fr Textausgabe
	Send_Command(TERMINAL_SETCURSOR);
// Bestimmung der Textposition	
	iValue = (x2-x1) / 2;
	buffer = caption;
	while (*buffer++) 
		iValue-=3;
	if (iValue > 0)	
		x1+=iValue;	
	iValue = y1;
	if (y2-y1 < 25)
		Send_Coordinates(x1, y1+5);
	else
	{
		iValue = (iValue + y2) / 2 - 5;
		y1 = iValue;
		Send_Coordinates(x1, y1);
	}	
	Send_String(caption);
	Send_Command(MODE_END_INTERMEDIATE);
}



void Send_Keyboard_Button_Text(const char *caption, unsigned char Button_Number, unsigned char force_switch)
{
	unsigned char x, y;
	
	if (Button_Number <= 35)
	{
		if (force_switch != 0)
		{
			Send_Command(KEYBOARD_SWITCHTERMINAL);
			Send_Command(TERMINAL_BKCOLOR);
			Send_Value(WindowBkColor);	// Hintergrundfarbe fr Textausgabe
		}	
		Send_Command(TERMINAL_SETCURSOR);
	
		y = Button_Number / 8;
		x = Button_Number - y*8;
	
		x=x*32+12;
		y=y*42+54;
		
		Send_Coordinates(x, y);
		Send_String(caption);
	
		if (force_switch != 0)
			Send_Command(MODE_END_INTERMEDIATE);
	}		
}
