#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include "param.h"
#include "global.h"
#include "tv.h"

unsigned char Y_Count, Sync_Count;	//Nicht benutzen, werden vom TV Generator benutzt !

const fram pixel = (fram)0x0400;

void tv_init(void)
{
	cli();

	MCUCR=1<<SRE;		//Enable XMEM

	DDRE=255;
	ICR1 =1024;
	OCR1B=SyncWidth;
	OCR1A=XStart-60;
	TCCR1A=(1<<COM1B1)|(1<<COM1B0)|(1<<WGM11)|(1<<WGM10);
	TCCR1B=(1<<WGM12)|1;		//10bit PWM bei 16MHz: 15,625kHz

	TIMSK=(1<<OCIE1A);
	TIFR=(1<<OCF1A);
	sei();
}

char CheckChar(unsigned char zeichen)		// Sonderzeichenbehandlung
{
	if (zeichen > 127)	
	{
		if (zeichen == 196)
			return 95;	
		else if (zeichen == 214)
			return 96;
		else if (zeichen == 220)
			return 97;
		else if (zeichen == 223)
			return 98;
		else if (zeichen == 228)
			return 99;
		else if (zeichen == 246)
			return 100;
		else if (zeichen == 252)
			return 101;	
		else
			return 0;
	}
	else if (zeichen < 32)
		return 0;
	else
		return zeichen-32;
}



void tv_writechar (unsigned char x, unsigned char y, unsigned char textfarbe, unsigned int hintergrundfarbe, unsigned char zeichen)
{	unsigned char wert,i,j;

	if (y>252)
		return;
		
	zeichen = CheckChar(zeichen);

	for(i=0; i<8; i++)
	{	wert=pgm_read_byte(&Font[zeichen][i]);
		for(j=0; j<6; j++)
		{	if (wert&32)
				pixel[y+i][x+j]=textfarbe;
			else // if (hintergrundfarbe<256)
				pixel[y+i][x+j]=hintergrundfarbe;
			wert<<=1;
		 }
	}
}

void tv_writechar2x (unsigned char x, unsigned char y, unsigned char textfarbe, unsigned int hintergrundfarbe, unsigned char zeichen)
{	
	unsigned char wert,i,j;

	if (y>252)
		return;
		
	zeichen = CheckChar(zeichen);	

	for(i=0; i<8; i++)
	{	wert=pgm_read_byte(&Font[zeichen][i]);
		for(j=0; j<6; j++)
		{	if (wert&32)
			{
				pixel[y+2*i][x+j]=textfarbe;
				pixel[y+2*i+1][x+j]=textfarbe;
			}
			else // if (hintergrundfarbe<256)
			{
				pixel[y+2*i][x+j]=hintergrundfarbe;
				pixel[y+2*i+1][x+j]=hintergrundfarbe;
			}	
			wert<<=1;
		 }
	}
}


void tv_writechar3x (unsigned char x, unsigned char y, unsigned char textfarbe, unsigned int hintergrundfarbe, unsigned char zeichen)
{	
	unsigned char wert,i,j;

	if (y>252)
		return;
		
	zeichen = CheckChar(zeichen);

	for(i=0; i<8; i++)
	{	wert=pgm_read_byte(&Font[zeichen][i]);
		for(j=0; j<6; j++)
		{	if (wert&32)
			{
				pixel[y+3*i][x+2*j]=textfarbe;
				pixel[y+3*i][x+2*j+1]=textfarbe;
				pixel[y+3*i+1][x+2*j]=textfarbe;
				pixel[y+3*i+1][x+2*j+1]=textfarbe;
				pixel[y+3*i+2][x+2*j]=textfarbe;
				pixel[y+3*i+2][x+2*j+1]=textfarbe;
			}
			else // if (hintergrundfarbe<256)
			{
				pixel[y+3*i][x+2*j]=hintergrundfarbe;
				pixel[y+3*i][x+2*j+1]=hintergrundfarbe;
				pixel[y+3*i+1][x+2*j]=hintergrundfarbe;
				pixel[y+3*i+1][x+2*j+1]=hintergrundfarbe;
				pixel[y+3*i+2][x+2*j]=hintergrundfarbe;
				pixel[y+3*i+2][x+2*j+1]=hintergrundfarbe;
			}	
			wert<<=1;
		 }
	}
}

void tv_setpixel (unsigned char x, unsigned char y, unsigned char farbe)
{	if (y>=252)
		return;
	pixel[y][x]=farbe;
}

void tv_clear(unsigned char farbe)
{	
	unsigned char x=XSIZE,y=YSIZE;
	while (y--)
		while (x--)
			pixel[y][x]=farbe;
}


void bresenham_line (unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned char farbe)
{	signed int dx,dy,x,y;
	signed char sdx,sdy;
	unsigned char i,px,py;

	dx=(signed int)x2-x1;      /* the horizontal distance of the line */
	dy=(signed int)y2-y1;      /* the vertical distance of the line */
	if (dx==0)
		sdx=0;
	if (dx>0)
		sdx=1;
	else
	{  	sdx=-1;
		dx=-dx;
	}
	if (dy==0)
		sdy=0;
	if (dy>0)
		sdy=1;
	else
	{	sdy=-1;
		dy=-dy;
	}
	x=dx*2;
	y=dy*2;
	px=x1;
	py=y1;
	tv_setpixel (px,py,farbe);

	if (dx>=dy) /* the line is more horizontal than vertical */
	{	for(i=0;i<dx;i++)
		{	y+=dy;
			if (y>=dx)
			{	y-=dx;
				py+=sdy;
			}
			px+=sdx;
			tv_setpixel (px,py,farbe);
		}
	}
	else /* the line is more vertical than horizontal */
	{	for(i=0;i<dy;i++)
		{	x+=dx;
			if (x>=dy)
			{	x-=dy;
				px+=sdx;
			}
			py+=sdy;
			tv_setpixel (px,py,farbe);
		}
	}
}

void h_line (unsigned char x1, unsigned char x2, unsigned char y, unsigned char farbe)
{unsigned int x;
	for (x=x1; x<=x2; x++)
		tv_setpixel (x,y,farbe);
}

void v_line (unsigned char x, unsigned char y1, unsigned char y2, unsigned char farbe)
{unsigned int y;
	for (y=y1; y<=y2; y++)
		tv_setpixel (x,y,farbe);
}

void tv_line (unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned char farbe)
{   unsigned int temp;

	if (y1==y2)
	{	if (x1>x2)
		{	temp=x1;
			x1=x2;
			x2=temp;
		}
		h_line (x1, x2, y1,farbe);
	}
	else if (x1==x2)
	{	if (y1>y2)
		{	temp=y1;
			y1=y2;
			y2=temp;
		}
		v_line (x1, y1, y2,farbe);
	}
	else
		bresenham_line (x1, y1, x2, y2, farbe);
}
